@@ -127,7 +127,7 @@ class RGBMapper(Callback.Callbacks):
             bpp = 8
         self.bpp = bpp
         # maximum value that we can generate in this range
-        self.maxc = 2 ** self.bpp - 1
+        self.maxc = int(2 ** self.bpp - 1)
         # data size per pixel band in the output RGB array
         self._set_dtype()
 
@@ -143,6 +143,11 @@ class RGBMapper(Callback.Callbacks):
         for name in ('changed', ):
             self.enable_callback(name)
 
+        self.suppress_changed = self.suppress_callback('changed', 'last')
+
+        carr = self.t_.get('color_array', None)
+        sarr = self.t_.get('shift_array', None)
+
         cm_name = self.t_.get('color_map', 'gray')
         self.set_color_map(cm_name)
 
@@ -150,14 +155,12 @@ class RGBMapper(Callback.Callbacks):
         self.set_intensity_map(im_name)
 
         # Initialize color array
-        carr = self.t_.get('color_array', None)
         if carr is not None:
             self.set_carr(carr)
         else:
             self.calc_cmap()
 
         # Initialize shift array
-        sarr = self.t_.get('shift_array', None)
         if sarr is not None:
             self.set_sarr(sarr)
         else:
@@ -178,7 +181,7 @@ class RGBMapper(Callback.Callbacks):
         be 16, etc.
         """
         self.bpp = bpp
-        self.maxc = 2 ** self.bpp - 1
+        self.maxc = int(2 ** self.bpp - 1)
         self._set_dtype()
         self.calc_cmap()
         self.recalc(callback=False)
@@ -201,11 +204,12 @@ class RGBMapper(Callback.Callbacks):
         any callbacks associated with this change will be invoked.
         """
         self.cmap = cmap
-        self.calc_cmap()
-        # TEMP: ignore passed callback parameter
-        # callback=False in the following because we don't want to
-        # recursively invoke set_cmap()
-        self.t_.set(color_map=cmap.name, callback=False)
+        with self.suppress_changed:
+            self.calc_cmap()
+            # TEMP: ignore passed callback parameter
+            # callback=False in the following because we don't want to
+            # recursively invoke set_cmap()
+            self.t_.set(color_map=cmap.name, callback=False)
 
     def get_cmap(self):
         """
@@ -232,9 +236,10 @@ class RGBMapper(Callback.Callbacks):
         self.t_.set(color_array=carr)
 
     def restore_cmap(self, callback=True):
-        self.reset_sarr(callback=False)
-        # TEMP: ignore passed callback parameter
-        self.calc_cmap()
+        with self.suppress_changed:
+            self.reset_sarr(callback=False)
+            # TEMP: ignore passed callback parameter
+            self.calc_cmap()
 
     def reset_cmap(self):
         self.recalc()
@@ -244,6 +249,7 @@ class RGBMapper(Callback.Callbacks):
         Return a tuple of (R, G, B) values in the 0-maxc range associated
         mapped by the value of `index`.
         """
+        index = int(index)
         return tuple(self.arr[index])
 
     def get_rgbval(self, index):
@@ -251,9 +257,10 @@ class RGBMapper(Callback.Callbacks):
         Return a tuple of (R, G, B) values in the 0-maxc range associated
         mapped by the value of `index`.
         """
+        index = int(index)
         assert (index >= 0) and (index <= self.maxc), \
             RGBMapError("Index must be in range 0-%d !" % (self.maxc))
-        index = self.sarr[index].clip(0, self.maxc)
+        index = int(self.sarr[index].clip(0, self.maxc))
         return (self.arr[0][index],
                 self.arr[1][index],
                 self.arr[2][index])
@@ -274,11 +281,12 @@ class RGBMapper(Callback.Callbacks):
         """
         self.imap = imap
         self.calc_imap()
-        # TEMP: ignore passed callback parameter
-        self.recalc()
-        # callback=False in the following because we don't want to
-        # recursively invoke set_imap()
-        self.t_.set(intensity_map=imap.name, callback=False)
+        with self.suppress_changed:
+            # TEMP: ignore passed callback parameter
+            self.recalc()
+            # callback=False in the following because we don't want to
+            # recursively invoke set_imap()
+            self.t_.set(intensity_map=imap.name, callback=False)
 
     def get_imap(self):
         """
@@ -407,12 +415,12 @@ class RGBMapper(Callback.Callbacks):
         # but clip as a precaution
         # See NOTE [A]: idx is always an array calculated in the caller and
         #    discarded afterwards
-        # idx = idx.clip(0, self.maxc)
+        # idx = idx.clip(0, self.maxc).astype(np.uint, copy=False)
         idx.clip(0, self.maxc, out=idx)
 
         # run it through the shift array and clip the result
         # See NOTE [A]
-        # idx = self.sarr[idx].clip(0, self.maxc)
+        # idx = self.sarr[idx].clip(0, self.maxc).astype(np.uint, copy=False)
         idx = self.sarr[idx]
         idx.clip(0, self.maxc, out=idx)
 
@@ -501,7 +509,7 @@ class RGBMapper(Callback.Callbacks):
         iscale_x = float(old_wd) / float(new_wd)
 
         xi = (xi * iscale_x).astype(np.uint, copy=False)
-        xi = xi.clip(0, old_wd - 1)
+        xi = xi.clip(0, old_wd - 1).astype(np.uint, copy=False)
         newdata = sarr[xi]
         return newdata
 
