@@ -17,6 +17,19 @@ file LICENSE.txt in the top-level directory for details.
 from ._astropy_init import *  # noqa
 # ----------------------------------------------------------------------------
 
+import sys
+
+__minimum_python_version__ = '3.5'
+
+
+class UnsupportedPythonError(Exception):
+    pass
+
+
+# This is the same check as the one in setup.cfg
+if sys.version_info < tuple((int(val) for val in __minimum_python_version__.split('.'))):
+    raise UnsupportedPythonError("Ginga does not support Python < {}".format(__minimum_python_version__))
+
 # For egg_info test builds to pass, put package imports here.
 if not _ASTROPY_SETUP_:  # noqa
     #from example_mod import *
