@@ -5,7 +5,6 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
-from __future__ import absolute_import, division, print_function
 
 import math
 import numpy as np
@@ -20,11 +19,12 @@ from ginga.misc import Bunch
 from ginga.util import wcs
 from ginga.util.wcs import raDegToString, decDegToString
 
-from .mixins import OnePointMixin, TwoPointMixin, OnePointOneRadiusMixin
+from .mixins import (OnePointMixin, TwoPointMixin, OnePointOneRadiusMixin,
+                     OnePointTwoRadiusMixin)
 from .layer import CompoundObject
 
 __all__ = ['Ruler', 'Compass', 'Crosshair', 'AnnulusMixin', 'Annulus',
-           'WCSAxes']
+           'Annulus2R', 'WCSAxes']
 
 
 class Ruler(TwoPointMixin, CanvasObjectBase):
@@ -76,7 +76,7 @@ class Ruler(TwoPointMixin, CanvasObjectBase):
                   description="Units for text distance (default: arcmin)"),
             Param(name='font', type=str, default='Sans Serif',
                   description="Font family for text"),
-            Param(name='fontsize', type=int, default=None,
+            Param(name='fontsize', type=float, default=None,
                   min=8, max=72,
                   description="Font size of text (default: vary by scale)"),
             Param(name='showcap', type=_bool,
@@ -286,7 +286,7 @@ class Compass(OnePointOneRadiusMixin, CanvasObjectBase):
                   description="Opacity of outline"),
             Param(name='font', type=str, default='Sans Serif',
                   description="Font family for text"),
-            Param(name='fontsize', type=int, default=None,
+            Param(name='fontsize', type=float, default=None,
                   min=8, max=72,
                   description="Font size of text (default: vary by scale)"),
             Param(name='showcap', type=_bool,
@@ -477,9 +477,12 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
                   description="Color of text annotation"),
             Param(name='font', type=str, default='Sans Serif',
                   description="Font family for text"),
-            Param(name='fontsize', type=int, default=None,
+            Param(name='fontsize', type=float, default=None,
                   min=8, max=72,
                   description="Font size of text (default: vary by scale)"),
+            Param(name='fontscale', type=_bool,
+                  default=True, valid=[False, True],
+                  description="Scale font with scale of viewer"),
             Param(name='format', type=str, default='xy',
                   valid=['xy', 'value', 'coords'],
                   description="Format for text annotation (default: xy)"),
@@ -492,17 +495,21 @@ class Crosshair(OnePointMixin, CanvasObjectBase):
     def __init__(self, x, y, color='green',
                  linewidth=1, alpha=1.0, linestyle='solid',
                  text=None, textcolor='yellow',
-                 fontsize=None, font='Sans Serif', format='xy',
-                 **kwdargs):
+                 fontsize=10.0, font='Sans Serif', fontscale=True,
+                 format='xy', **kwdargs):
         self.kind = 'crosshair'
         points = np.asarray([(x, y)], dtype=np.float)
         CanvasObjectBase.__init__(self, color=color, alpha=alpha,
                                   linewidth=linewidth, linestyle=linestyle,
                                   text=text, textcolor=textcolor,
                                   fontsize=fontsize, font=font,
+                                  fontscale=fontscale,
                                   points=points, format=format, **kwdargs)
         OnePointMixin.__init__(self)
 
+        self.fontsize_min = 8.0
+        self.fontsize_max = 14.0
+
     def select_contains_pt(self, viewer, pt):
         p0 = self.crdmap.to_data((self.x, self.y))
         return self.within_radius(viewer, pt, p0, self.cap_radius)
@@ -672,6 +679,7 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
 
         self.editable = True
         self.opaque = True
+        self.atype = atype
         self.kind = 'annulus'
 
     def get_edit_points(self, viewer):
@@ -735,6 +743,211 @@ class Annulus(AnnulusMixin, OnePointOneRadiusMixin, CompoundObject):
         self.set_data_points([dst_pt])
 
 
+class Annulus2R(AnnulusMixin, OnePointTwoRadiusMixin, CompoundObject):
+    """
+    Special compound object to handle annulus shape that
+    consists of two objects, (one center point, plus two radii).
+
+    Examples
+    --------
+    >>> tag = canvas.add(Annulus2R(100, 200, 10, 20, width=5, atype='box'))
+    >>> obj = canvas.get_object_by_tag(tag)
+    >>> arr_masked = image.cutout_shape(obj)
+
+    """
+    @classmethod
+    def get_params_metadata(cls):
+        return [
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
+            Param(name='x', type=float, default=0.0, argpos=0,
+                  description="X coordinate of center of object"),
+            Param(name='y', type=float, default=0.0, argpos=1,
+                  description="Y coordinate of center of object"),
+            Param(name='xradius', type=float, default=1.0, argpos=2,
+                  min=0.0,
+                  description="Inner X radius of annulus"),
+            Param(name='yradius', type=float, default=1.0, argpos=2,
+                  min=0.0,
+                  description="Inner Y radius of annulus"),
+            Param(name='xwidth', type=float, default=None,
+                  min=0.0,
+                  description="Width in X of annulus"),
+            Param(name='ywidth', type=float, default=None,
+                  min=0.0,
+                  description="Width in Y of annulus"),
+            Param(name='atype', type=str, default='ellipse',
+                  valid=['ellipse', 'box'],
+                  description="Type of annulus"),
+            Param(name='linewidth', type=int, default=1,
+                  min=1, max=20, widget='spinbutton', incr=1,
+                  description="Width of outline"),
+            Param(name='linestyle', type=str, default='solid',
+                  valid=['solid', 'dash'],
+                  description="Style of outline (default solid)"),
+            Param(name='color',
+                  valid=colors_plus_none, type=_color, default='yellow',
+                  description="Color of outline"),
+            Param(name='alpha', type=float, default=1.0,
+                  min=0.0, max=1.0, widget='spinfloat', incr=0.05,
+                  description="Opacity of outline"),
+            Param(name='rot_deg', type=float, default=0.0,
+                  min=-359.999, max=359.999, widget='spinfloat', incr=1.0,
+                  description="Rotation about center of object"),
+        ]
+
+    @classmethod
+    def idraw(cls, canvas, cxt):
+        xradius, yradius = abs(cxt.start_x - cxt.x), abs(cxt.start_y - cxt.y)
+        return cls(cxt.start_x, cxt.start_y, xradius, yradius, **cxt.drawparams)
+
+    def __init__(self, x, y, xradius, yradius, xwidth=None,
+                 ywidth=None, atype='ellipse', color='yellow',
+                 linewidth=1, linestyle='solid', alpha=1.0,
+                 rot_deg=0.0, **kwdargs):
+
+        if xwidth is None:
+            # default X width is 15% of X radius
+            xwidth = 0.15 * xradius
+        if ywidth is None:
+            # default Y width is X width
+            ywidth = xwidth
+        oxradius = xradius + xwidth
+        oyradius = yradius + ywidth
+
+        if oxradius < xradius or oyradius < yradius:
+            raise ValueError('Outer boundary < inner boundary')
+
+        coord = kwdargs.get('coord', None)
+
+        klass = get_canvas_type(atype)
+        obj1 = klass(x, y, xradius, yradius, color=color,
+                     linewidth=linewidth,
+                     linestyle=linestyle, alpha=alpha,
+                     coord=coord, rot_deg=rot_deg)
+        obj1.editable = False
+
+        obj2 = klass(x, y, oxradius, oyradius, color=color,
+                     linewidth=linewidth,
+                     linestyle=linestyle, alpha=alpha,
+                     coord=coord, rot_deg=rot_deg)
+        obj2.editable = False
+
+        points = np.asarray([(x, y)], dtype=np.float)
+
+        CompoundObject.__init__(self, obj1, obj2,
+                                points=points, xradius=xradius, yradius=yradius,
+                                xwidth=xwidth, ywidth=ywidth, color=color,
+                                linewidth=linewidth, linestyle=linestyle,
+                                alpha=alpha, rot_deg=rot_deg, **kwdargs)
+        OnePointTwoRadiusMixin.__init__(self)
+
+        self.editable = True
+        self.opaque = True
+        self.atype = atype
+        self.kind = 'annulus2r'
+
+    def get_edit_points(self, viewer):
+        move_pt, scale_pt, rotate_pt = self.get_move_scale_rotate_pts(viewer)
+
+        points = (self.crdmap.offset_pt((self.x, self.y),
+                                        (self.xradius, 0)),
+                  self.crdmap.offset_pt((self.x, self.y),
+                                        (0, self.yradius)),
+                  self.crdmap.offset_pt((self.x, self.y),
+                                        (self.xradius + self.xwidth, 0)),
+                  self.crdmap.offset_pt((self.x, self.y),
+                                        (0, self.yradius + self.ywidth)),
+                  )
+        points = self.get_data_points(points=points)
+        return [move_pt,    # location
+                Point(*points[0]),  # adj inner X radius
+                Point(*points[1]),  # adj inner Y radius
+                Point(*points[2]),  # adj X width
+                Point(*points[3]),  # adj Y width
+                scale_pt,
+                rotate_pt,
+                ]
+
+    def setup_edit(self, detail):
+        detail.center_pos = self.get_center_pt()
+        detail.xradius = self.xradius
+        detail.yradius = self.yradius
+        detail.xwidth = self.xwidth
+        detail.ywidth = self.ywidth
+
+    def set_edit_point(self, i, pt, detail):
+        if i == 0:
+            self.move_to_pt(pt)
+        elif i == 1:
+            # Adjust inner X radius
+            scale_x, scale_y = self.calc_dual_scale_from_pt(pt, detail)
+            self.xradius = detail.xradius * scale_x
+            #scalef = self.calc_scale_from_pt(pt, detail)
+            # inner obj radius control pt
+            #self.xradius = detail.xradius * scalef
+        elif i == 2:
+            # Adjust inner Y radius
+            scale_x, scale_y = self.calc_dual_scale_from_pt(pt, detail)
+            self.yradius = detail.yradius * scale_y
+            #scalef = self.calc_scale_from_pt(pt, detail)
+            #self.yradius = detail.yradius * scalef
+        elif i == 3:
+            # Adjust X width
+            scale_x, scale_y = self.calc_dual_scale_from_pt(pt, detail)
+            xwidth = detail.xwidth * scale_x
+            # outer obj radius control pt--calculate new width
+            assert xwidth > 0, ValueError("Must have a positive width")
+            self.xwidth = xwidth
+        elif i == 4:
+            # Adjust Y width
+            scale_x, scale_y = self.calc_dual_scale_from_pt(pt, detail)
+            ywidth = detail.ywidth * scale_y
+            # outer obj radius control pt--calculate new width
+            assert ywidth > 0, ValueError("Must have a positive width")
+            self.ywidth = ywidth
+        elif i == 5:
+            # Adjust overall scale
+            scalef = self.calc_scale_from_pt(pt, detail)
+            self.xradius = detail.xradius * scalef
+            self.yradius = detail.yradius * scalef
+        elif i == 6:
+            # Adjust rotation
+            delta_deg = self.calc_rotation_from_pt(pt, detail)
+            self.rotate_by_deg([delta_deg])
+        else:
+            raise ValueError("No point corresponding to index %d" % (i))
+
+        self.sync_state()
+
+    def sync_state(self):
+        """Called to synchronize state (e.g. when parameters have changed).
+        """
+        oxradius = self.xradius + self.xwidth
+        oyradius = self.yradius + self.ywidth
+        if oxradius < self.xradius or oyradius < self.yradius:
+            raise ValueError('Outer boundary < inner boundary')
+
+        d = dict(points=self.points, xradius=self.xradius,
+                 yradius=self.yradius, color=self.color,
+                 linewidth=self.linewidth, linestyle=self.linestyle,
+                 alpha=self.alpha, rot_deg=self.rot_deg)
+
+        # update inner object
+        self.objects[0].__dict__.update(d)
+
+        # update outer object
+        d['xradius'] = oxradius
+        d['yradius'] = oyradius
+        self.objects[1].__dict__.update(d)
+
+    def move_to_pt(self, dst_pt):
+        super(Annulus2R, self).move_to_pt(dst_pt)
+
+        self.set_data_points([dst_pt])
+
+
 class WCSAxes(CompoundObject):
     """
     Special compound object to draw WCS axes.
@@ -756,7 +969,7 @@ class WCSAxes(CompoundObject):
                   description="Opacity of grid and text"),
             Param(name='font', type=str, default='Sans Serif',
                   description="Font family for text"),
-            Param(name='fontsize', type=int, default=8,
+            Param(name='fontsize', type=float, default=8.0,
                   min=8, max=72,
                   description="Font size of text (default: 8)"),
         ]
@@ -795,6 +1008,13 @@ class WCSAxes(CompoundObject):
         self._cur_swap = swapxy
 
         if not isinstance(image, AstroImage) or not image.has_valid_wcs():
+            self.logger.debug(
+                'WCSAxes can only be displayed for AstroImage with valid WCS')
+            return []
+
+        min_imsize = min(image.width, image.height)
+        if min_imsize <= 0:
+            self.logger.debug('Cannot draw WCSAxes on image with 0 dim')
             return []
 
         # Approximate bounding box in RA/DEC space
@@ -804,7 +1024,8 @@ class WCSAxes(CompoundObject):
             radec = image.wcs.datapt_to_system(
                 [[0, 0], [0, ymax], [xmax, 0], [xmax, ymax]],
                 naxispath=image.naxispath)
-        except Exception:
+        except Exception as e:
+            self.logger.warning('WCSAxes failed: {}'.format(str(e)))
             return []
         ra_min, dec_min = radec.ra.min().deg, radec.dec.min().deg
         ra_max, dec_max = radec.ra.max().deg, radec.dec.max().deg
@@ -818,7 +1039,6 @@ class WCSAxes(CompoundObject):
         dec_arr = np.arange(dec_min + d_dec, dec_max - d_ra * 0.5, d_dec)
 
         # RA/DEC step size for each vector
-        min_imsize = min(image.width, image.height)
         d_ra_step = ra_size * self._pix_res / min_imsize
         d_dec_step = dec_size * self._pix_res / min_imsize
 
@@ -843,7 +1063,8 @@ class WCSAxes(CompoundObject):
 
         try:
             pts = image.wcs.wcspt_to_datapt(crds, naxispath=image.naxispath)
-        except Exception:
+        except Exception as e:
+            self.logger.warning('WCSAxes failed: {}'.format(str(e)))
             return []
 
         # Don't draw outside image area
@@ -851,6 +1072,12 @@ class WCSAxes(CompoundObject):
                 (pts[:, 1] >= 0) & (pts[:, 1] < image.height))
         pts = pts[mask]
 
+        if len(pts) == 0:
+            self.logger.debug(
+                'All WCSAxes coords ({}) out of bound in {}x{} '
+                'image'.format(crds, image.width, image.height))
+            return []
+
         path_obj = Path(
             points=pts, coords='data', linewidth=self.linewidth,
             linestyle=self.linestyle, color=self.color,
@@ -967,6 +1194,6 @@ class WCSAxes(CompoundObject):
 
 register_canvas_types(dict(ruler=Ruler, compass=Compass,
                            crosshair=Crosshair, annulus=Annulus,
-                           wcsaxes=WCSAxes))
+                           annulus2r=Annulus2R, wcsaxes=WCSAxes))
 
 # END
