@@ -30,7 +30,7 @@ class ColorBar(CanvasObjectBase):
                   description="Show the range in the colorbar"),
             Param(name='font', type=str, default='Sans Serif',
                   description="Font family for text"),
-            Param(name='fontsize', type=int, default=8,
+            Param(name='fontsize', type=float, default=8.0,
                   min=8, max=72,
                   description="Font size of text (default: 8)"),
             Param(name='linewidth', type=int, default=1,
@@ -220,7 +220,7 @@ class DrawableColorBar(Rectangle):
                   description="Show the range in the colorbar"),
             Param(name='font', type=str, default='Sans Serif',
                   description="Font family for text"),
-            Param(name='fontsize', type=int, default=8,
+            Param(name='fontsize', type=float, default=8,
                   min=8, max=72,
                   description="Font size of text (default: 8)"),
             Param(name='linewidth', type=int, default=1,
@@ -407,7 +407,7 @@ class ModeIndicator(CanvasObjectBase):
                   description="Offset in pixels from the right and bottom of the window"),
             Param(name='font', type=str, default='Sans Serif',
                   description="Font family for text"),
-            Param(name='fontsize', type=int, default=None,
+            Param(name='fontsize', type=float, default=None,
                   min=8, max=72,
                   description="Font size of text (default: vary by scale)"),
             Param(name='color',
