@@ -1,4 +1,3 @@
-from __future__ import print_function
 
 import sys
 import os
@@ -9,7 +8,7 @@ from bokeh.models.widgets import TextInput
 
 from ginga.web.bokehw import ImageViewBokeh as ib
 from ginga.misc import log
-from ginga.AstroImage import AstroImage
+from ginga.util.loader import load_data
 
 
 def main(options, args):
@@ -24,8 +23,7 @@ def main(options, args):
     viewer.set_figure(fig)
 
     def load_file(path):
-        image = AstroImage(logger)
-        image.load_file(path)
+        image = load_data(path, logger=logger)
         viewer.set_image(image)
 
     def load_file_cb(attr_name, old_val, new_val):
@@ -47,25 +45,22 @@ def main(options, args):
 if __name__ == "__main__":
 
     # Parse command line options
-    from optparse import OptionParser
+    from argparse import ArgumentParser
 
-    usage = "usage: %prog [options] cmd [args]"
-    optprs = OptionParser(usage=usage, version=('%%prog'))
+    argprs = ArgumentParser()
 
-    optprs.add_option("--debug", dest="debug", default=False, action="store_true",
-                      help="Enter the pdb debugger on main()")
-    optprs.add_option("-d", "--indir", dest="indir", metavar="DIR",
-                      default=os.environ['HOME'],
-                      help="Look in DIR for files")
-    optprs.add_option("--opencv", dest="use_opencv", default=False,
-                      action="store_true",
-                      help="Use OpenCv acceleration")
-    optprs.add_option("--profile", dest="profile", action="store_true",
-                      default=False,
-                      help="Run the profiler on main()")
-    log.addlogopts(optprs)
+    argprs.add_argument("--debug", dest="debug", default=False,
+                        action="store_true",
+                        help="Enter the pdb debugger on main()")
+    argprs.add_argument("-d", "--indir", dest="indir", metavar="DIR",
+                        default=os.environ['HOME'],
+                        help="Look in DIR for files")
+    argprs.add_argument("--profile", dest="profile", action="store_true",
+                        default=False,
+                        help="Run the profiler on main()")
+    log.addlogopts(argprs)
 
-    (options, args) = optprs.parse_args(sys.argv[1:])
+    (options, args) = argprs.parse_known_args(sys.argv[1:])
 
     # Are we debugging this?
     if options.debug:
