@@ -24,6 +24,10 @@ from gi.repository import Pango  # noqa
 ginga.toolkit.use('gtk3')
 
 
+DND_TARGET_TYPE_TEXT = 0
+DND_TARGET_TYPE_URIS = 1
+
+
 class WidgetMask(object):
     def __init__(self, *args):
         self.cb_fn = None
@@ -162,7 +166,10 @@ class VScale(WidgetMask, Gtk.VScale):
         super(VScale, self).set_value(newval)
 
 
-class ComboBoxMixin(object):
+class ComboBox(WidgetMask, Gtk.ComboBox):
+    def __init__(self, *args, **kwdargs):
+        WidgetMask.__init__(self)
+        Gtk.ComboBox.__init__(self, *args, **kwdargs)
 
     def set_active(self, newval):
         oldval = self.get_active()
@@ -206,18 +213,6 @@ class ComboBoxMixin(object):
                 return
 
 
-class ComboBox(WidgetMask, Gtk.ComboBox, ComboBoxMixin):
-    def __init__(self, *args, **kwdargs):
-        WidgetMask.__init__(self)
-        Gtk.ComboBox.__init__(self, *args, **kwdargs)
-
-
-## class ComboBoxEntry(WidgetMask, Gtk.ComboBoxEntry, ComboBoxMixin):
-##     def __init__(self, *args, **kwdargs):
-##         WidgetMask.__init__(self)
-##         Gtk.ComboBoxEntry.__init__(self, *args, **kwdargs)
-
-
 class Notebook(WidgetMask, Gtk.Notebook):
     def __init__(self, *args, **kwdargs):
         WidgetMask.__init__(self)
@@ -293,7 +288,7 @@ class MDISubWindow(Callback.Callbacks):
 
         evbox = Gtk.EventBox()
         evbox.add(label)
-        evbox.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse("gray90"))
+        modify_bg(evbox, "gray90")
         self.label = evbox
         self.evbox = evbox
         hbox.pack_start(evbox, True, True, 2)
@@ -313,7 +308,7 @@ class MDISubWindow(Callback.Callbacks):
         frame.set_size_request(self.width, self.height)
         frame.props.visible_window = True
         frame.set_border_width(0)
-        frame.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse("gray70"))
+        modify_bg(frame, "gray70")
         self.frame = frame
 
         frame.add(vbox)
@@ -362,7 +357,7 @@ class MDIWidget(Gtk.Layout):
         self.connect("button_press_event", self.button_press_event)
         self.connect("button_release_event", self.button_release_event)
 
-        self.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse("gray50"))
+        modify_bg(self, "gray50")
 
     def append_page(self, widget, label):
 
@@ -738,8 +733,11 @@ class MDIWidget(Gtk.Layout):
 class FileSelection(object):
 
     def __init__(self, parent_w, action=Gtk.FileChooserAction.OPEN,
-                 title="Select a file"):
+                 title="Select a file", all_at_once=False):
+        # TODO: deprecate the functionality when all_at_once == False
+        # and make the default to be True
         self.parent = parent_w
+        self.all_at_once = all_at_once
         # Create a new file selection widget
         self.filew = Gtk.FileChooserDialog(title=title, action=action)
         self.filew.connect("destroy", self.close)
@@ -748,6 +746,7 @@ class FileSelection(object):
         else:
             self.filew.add_buttons(Gtk.STOCK_OPEN, 1, Gtk.STOCK_CANCEL, 0)
         self.filew.set_default_response(1)
+        self.filew.set_select_multiple(True)
         self.filew.connect("response", self.file_ok_sel)
 
         # Connect the cancel_button to destroy the widget
@@ -765,6 +764,8 @@ class FileSelection(object):
             self.filew.set_current_name(filename)
 
         self.filew.show()
+        # default size can be enormous
+        self.filew.resize(800, 600)
 
     # Get the selected filename
     def file_ok_sel(self, w, rsp):
@@ -772,8 +773,12 @@ class FileSelection(object):
         if rsp == 0:
             return
 
-        filepath = self.filew.get_filename()
-        self.cb(filepath)
+        paths = self.filew.get_filenames()
+        if self.all_at_once:
+            self.cb(paths)
+        else:
+            for path in paths:
+                self.cb(path)
 
     def close(self, widget):
         self.filew.hide()
@@ -961,6 +966,18 @@ def make_cursor(widget, iconpath, x, y):
     return Gdk.Cursor(display, pixbuf, x, y)
 
 
+def modify_bg(widget, color):
+    context = widget.get_style_context()
+    if color is not None:
+        context.add_class("custom_bg")
+        css_data = "*.custom_bg { background-image: none; background-color: %s; }" % (color)
+        css_provider = Gtk.CssProvider()
+        css_provider.load_from_data(css_data.encode())
+        context.add_provider(css_provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)
+    else:
+        context.remove_class("custom_bg")
+
+
 def set_default_style():
     style_provider = Gtk.CssProvider()
 
