@@ -4,15 +4,14 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 
+import sys
 import os
 import numpy as np
 
 from ginga.gtk3w import GtkHelp
-from ginga import Mixins, Bindings
+from ginga import ImageView, Mixins, Bindings
 from ginga.util.paths import icondir
-
-from ginga.cairow.ImageViewCairo import (ImageViewCairo as ImageView,
-                                         ImageViewCairoError as ImageViewError)
+from ginga.canvas import render
 
 from gi.repository import Gtk
 from gi.repository import Gdk
@@ -20,16 +19,16 @@ from gi.repository import GdkPixbuf
 import cairo
 
 
-class ImageViewGtkError(ImageViewError):
+class ImageViewGtkError(ImageView.ImageViewError):
     pass
 
 
-class ImageViewGtk(ImageView):
+class ImageViewGtk(ImageView.ImageViewBase):
 
     def __init__(self, logger=None, rgbmap=None, settings=None):
-        ImageView.__init__(self, logger=logger,
-                           rgbmap=rgbmap,
-                           settings=settings)
+        ImageView.ImageViewBase.__init__(self, logger=logger,
+                                         rgbmap=rgbmap,
+                                         settings=settings)
 
         imgwin = Gtk.DrawingArea()
         imgwin.connect("draw", self.draw_event)
@@ -44,6 +43,23 @@ class ImageViewGtk(ImageView):
         self.imgwin = imgwin
         self.imgwin.show_all()
 
+        self.t_.set_defaults(renderer='cairo')
+
+        # create our default double-buffered surface area that we copy
+        # to the widget
+        rect = self.imgwin.get_allocation()
+        x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+        arr = np.zeros((ht, wd, 4), dtype=np.uint8)
+        stride = cairo.ImageSurface.format_stride_for_width(cairo.FORMAT_ARGB32,
+                                                            wd)
+        self.surface = cairo.ImageSurface.create_for_data(arr,
+                                                          cairo.FORMAT_ARGB32,
+                                                          wd, ht, stride)
+        if sys.byteorder == 'little':
+            self.rgb_order = 'BGRA'
+        else:
+            self.rgb_order = 'ARGB'
+
         # see reschedule_redraw() method
         self._defer_task = GtkHelp.Timer()
         self._defer_task.add_callback('expired',
@@ -52,9 +68,38 @@ class ImageViewGtk(ImageView):
         self.msgtask.add_callback('expired',
                                   lambda timer: self.onscreen_message(None))
 
+        self.renderer = None
+        # Pick a renderer that can work with us
+        renderers = ['cairo', 'agg', 'pil', 'opencv']
+        preferred = self.t_['renderer']
+        if preferred in renderers:
+            renderers.remove(preferred)
+        self.possible_renderers = [preferred] + renderers
+        self.choose_best_renderer()
+
     def get_widget(self):
         return self.imgwin
 
+    def choose_renderer(self, name):
+        klass = render.get_render_class(name)
+        self.renderer = klass(self)
+
+        rect = self.imgwin.get_allocation()
+        x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+        self.configure_window(wd, ht)
+
+    def choose_best_renderer(self):
+        for name in self.possible_renderers:
+            try:
+                self.choose_renderer(name)
+                self.logger.info("best renderer available is '{}'".format(name))
+                return
+            except Exception as e:
+                self.logger.error("Error choosing renderer '{}': {}".format(name, e))
+                continue
+
+        raise ImageViewGtkError("No valid renderers available: {}".format(str(self.possible_renderers)))
+
     def get_plain_image_as_pixbuf(self):
         arr = self.getwin_array(order='RGB', dtype=np.uint8)
         pixbuf = GtkHelp.pixbuf_new_from_array(arr,
@@ -94,10 +139,11 @@ class ImageViewGtk(ImageView):
 
     def save_rgb_image_as_file(self, filepath, format='png', quality=90):
         pixbuf = self.get_rgb_image_as_pixbuf()
-        options = {}
+        options, values = [], []
         if format == 'jpeg':
-            options['quality'] = str(quality)
-        pixbuf.save(filepath, format, options)
+            options.append('quality')
+            values.append(str(quality))
+        pixbuf.savev(filepath, format, options, values)
 
     def reschedule_redraw(self, time_sec):
         self._defer_task.stop()
@@ -252,13 +298,16 @@ class ImageViewEvent(ImageViewGtk):
                           Gdk.EventMask.SCROLL_MASK)
 
         # Set up widget as a drag and drop destination
+        targets = Gtk.TargetList.new([])
+        targets.add_text_targets(GtkHelp.DND_TARGET_TYPE_TEXT)
+        targets.add_uri_targets(GtkHelp.DND_TARGET_TYPE_URIS)
+        imgwin.drag_dest_set(Gtk.DestDefaults.ALL, [],
+                             Gdk.DragAction.COPY)
+        imgwin.drag_dest_set_target_list(targets)
+
         imgwin.connect("drag-data-received", self.drop_event_cb)
         imgwin.connect("drag-motion", self.drag_motion_cb)
         imgwin.connect("drag-drop", self.drag_drop_cb)
-        self.TARGET_TYPE_TEXT = 0
-        imgwin.drag_dest_set(Gtk.DestDefaults.ALL, [],
-                             Gdk.DragAction.COPY)
-        imgwin.drag_dest_add_text_targets()
 
         # @$%&^(_)*&^ gnome!!
         self._keytbl = {
@@ -351,7 +400,7 @@ class ImageViewEvent(ImageViewGtk):
         except KeyError:
             return keyname
 
-    def get_keyTable(self):
+    def get_key_table(self):
         return self._keytbl
 
     def map_event(self, widget, event):
@@ -468,9 +517,9 @@ class ImageViewEvent(ImageViewGtk):
                                      data_x, data_y)
 
     def drag_drop_cb(self, widget, context, x, y, time):
-        self.logger.debug('drag_drop_cb')
+        self.logger.debug("drag_drop_cb initiated")
         # initiates a drop
-        success = delete = False  # noqa
+        success = False
         targets = context.list_targets()
         for mimetype in targets:
             if str(mimetype) in ("text/thumb", "text/plain", "text/uri-list"):
@@ -483,7 +532,7 @@ class ImageViewEvent(ImageViewGtk):
         return True
 
     def drag_motion_cb(self, widget, context, x, y, time):
-        self.logger.debug('drag_motion_cb')
+        self.logger.debug("drag_motion_cb initiated")
         # checks whether a drop is possible
         targets = context.list_targets()
         for mimetype in targets:
@@ -492,25 +541,36 @@ class ImageViewEvent(ImageViewGtk):
                 return True
 
         Gdk.drag_status(context, 0, time)
-        self.logger.debug('drag_motion_cb done')
+        self.logger.debug("drag_motion_cb done")
         return False
 
     def drop_event_cb(self, widget, context, x, y, selection, info, time):
-        self.logger.debug('drop_event')
-        if info != self.TARGET_TYPE_TEXT:
-            Gtk.drag_finish(context, False, False, time)
-            return False
+        self.logger.debug("drop event initiated")
 
-        buf = selection.get_text().strip()
-        if '\r\n' in buf:
-            paths = buf.split('\r\n')
+        if selection is None or selection.get_length() == 0:
+            return
+
+        paths = []
+        if info == GtkHelp.DND_TARGET_TYPE_TEXT:
+            # selection contains text
+            buf = selection.get_text().strip()
+            if '\r\n' in buf:
+                # windows
+                paths = buf.split('\r\n')
+            else:
+                paths = buf.split('\n')
+        elif info == GtkHelp.DND_TARGET_TYPE_URIS:
+            # selection contains URIs
+            paths = selection.get_uris()
         else:
-            paths = buf.split('\n')
+            self.logger.error("Dropped selection type ({}) is not recognized!".format(info))
+            return
+
         self.logger.debug("dropped filename(s): %s" % (str(paths)))
 
-        self.make_ui_callback('drag-drop', paths)
+        if len(paths) > 0:
+            self.make_ui_callback('drag-drop', paths)
 
-        Gtk.drag_finish(context, True, False, time)
         return True
 
 
@@ -534,7 +594,7 @@ class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
                                 settings=settings)
         Mixins.UIMixin.__init__(self)
 
-        self.ui_setActive(True)
+        self.ui_set_active(True)
 
         if bindmap is None:
             bindmap = ImageViewZoom.bindmapClass(self.logger)
@@ -557,6 +617,7 @@ class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
 
 
 class CanvasView(ImageViewZoom):
+    """A Ginga viewer for viewing 2D slices of image data."""
 
     def __init__(self, logger=None, settings=None, rgbmap=None,
                  bindmap=None, bindings=None):
@@ -614,18 +675,18 @@ class ScrolledView(Gtk.Table):
 
         self.viewer.add_callback('redraw', self._calc_scrollbars)
         self.viewer.add_callback('limits-set',
-                                 lambda v, l: self._calc_scrollbars(v))
+                                 lambda v, l: self._calc_scrollbars(v, 0))
 
-        self._calc_scrollbars(self.viewer)
+        self._calc_scrollbars(self.viewer, 0)
 
     def get_widget(self):
         return self
 
-    def _calc_scrollbars(self, viewer):
+    def _calc_scrollbars(self, viewer, whence):
         """Calculate and set the scrollbar handles from the pan and
         zoom positions.
         """
-        if self._scrolling:
+        if self._scrolling or whence > 0:
             return
 
         # flag that suppresses a cyclical callback
