@@ -8,7 +8,6 @@
 # import time
 
 from ginga.gtk3w import GtkHelp
-import ginga.util.six as six
 
 from ginga.misc import Callback, Bunch, LineHistory
 from functools import reduce
@@ -119,8 +118,9 @@ class WidgetBase(Callback.Callbacks):
         self.widget.set_size_request(width, height)
 
         # hackish way to allow the widget to be resized down again later
-        # NOTE: this causes some problems for sizing certain widgets
-        # GObject.idle_add(self.widget.set_size_request, -1, -1)
+        # NOTE: this may cause some problems for sizing certain widgets
+        if width > 0 and height > 0:
+            GObject.idle_add(self.widget.set_size_request, -1, -1)
 
     def get_font(self, font_family, point_size):
         font = GtkHelp.get_font(font_family, point_size)
@@ -182,7 +182,7 @@ class TextEntry(WidgetBase):
         self.widget.set_editable(tf)
 
     def set_font(self, font, size=10):
-        if isinstance(font, six.string_types):
+        if isinstance(font, str):
             font = self.get_font(font, size)
         self.widget.modify_font(font)
 
@@ -225,7 +225,7 @@ class TextEntrySet(WidgetBase):
         self.entry.set_editable(tf)
 
     def set_font(self, font, size=10):
-        if isinstance(font, six.string_types):
+        if isinstance(font, str):
             font = self.get_font(font, size)
         self.widget.modify_font(font)
 
@@ -310,7 +310,7 @@ class TextArea(WidgetBase):
         self.tw.set_editable(tf)
 
     def set_font(self, font, size=10):
-        if isinstance(font, six.string_types):
+        if isinstance(font, str):
             font = self.get_font(font, size)
         self.tw.modify_font(font)
 
@@ -382,13 +382,13 @@ class Label(WidgetBase):
         self.label.set_text(text)
 
     def set_font(self, font, size=10):
-        if isinstance(font, six.string_types):
+        if isinstance(font, str):
             font = self.get_font(font, size)
         self.label.modify_font(font)
 
     def set_color(self, fg=None, bg=None):
         if bg is not None:
-            self.evbox.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse(bg))
+            GtkHelp.modify_bg(self.evbox, bg)
         if fg is not None:
             self.label.modify_fg(Gtk.StateType.NORMAL, Gdk.color_parse(fg))
 
@@ -411,15 +411,14 @@ class ComboBox(WidgetBase):
     def __init__(self, editable=False):
         super(ComboBox, self).__init__()
 
-        if editable:
-            cb = GtkHelp.ComboBoxEntry()
-        else:
-            cb = GtkHelp.ComboBox()
+        cb = GtkHelp.ComboBox(has_entry=editable)
         liststore = Gtk.ListStore(GObject.TYPE_STRING)
         cb.set_model(liststore)
         cell = Gtk.CellRendererText()
         cb.pack_start(cell, True)
         cb.add_attribute(cell, 'text', 0)
+        if editable:
+            cb.set_entry_text_column(0)
         self.widget = cb
         self.widget.sconnect('changed', self._cb_redirect)
 
@@ -466,20 +465,38 @@ class ComboBox(WidgetBase):
     def clear(self):
         model = self.widget.get_model()
         model.clear()
+        if self.widget.get_has_entry():
+            entry = self.widget.get_entry()
+            entry.set_text('')
 
-    def show_text(self, text):
+    def set_text(self, text):
         model = self.widget.get_model()
         for i in range(len(model)):
             if model[i][0] == text:
                 self.widget.set_active(i)
                 return
 
+        if self.widget.get_has_entry():
+            entry = self.widget.get_child()
+            entry.set_text(text)
+
+    # to be deprecated someday
+    show_text = set_text
+
     def set_index(self, index):
         self.widget.set_active(index)
 
     def get_index(self):
         return self.widget.get_active()
 
+    def get_text(self):
+        if self.widget.get_has_entry():
+            entry = self.widget.get_child()
+            return entry.get_text()
+
+        idx = self.get_index()
+        return self.get_alpha(idx)
+
 
 class SpinBox(WidgetBase):
     def __init__(self, dtype=int):
@@ -752,9 +769,10 @@ class TreeView(WidgetBase):
         if self.dragable:
             tv = GtkHelp.MultiDragDropTreeView()
             # enable drag from this widget
-            toImage = [("text/plain", 0, 0)]
+            targets = [("text/plain", 0, GtkHelp.DND_TARGET_TYPE_TEXT),
+                       ("text/uri-list", 0, GtkHelp.DND_TARGET_TYPE_URIS)]
             tv.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK,
-                                        toImage, Gdk.DragAction.COPY)
+                                        targets, Gdk.DragAction.COPY)
             tv.connect("drag-data-get", self._start_drag)
         else:
             tv = Gtk.TreeView()
@@ -1010,8 +1028,8 @@ class TreeView(WidgetBase):
             model, iter1, iter2 = args[:3]
             bnch1 = model.get_value(iter1, 0)
             bnch2 = model.get_value(iter2, 0)
-            if isinstance(bnch1, six.string_types):
-                if isinstance(bnch2, six.string_types):
+            if isinstance(bnch1, str):
+                if isinstance(bnch2, str):
                     s1, s2 = bnch1.lower(), bnch2.lower()
                     if s1 < s2:
                         return -1
@@ -1019,7 +1037,7 @@ class TreeView(WidgetBase):
                         return 1
                 return 0
             val1, val2 = bnch1[idx], bnch2[idx]
-            if isinstance(val1, six.string_types):
+            if isinstance(val1, str):
                 val1, val2 = val1.lower(), val2.lower()
                 if val1 < val2:
                     return -1
@@ -1325,9 +1343,9 @@ class TabWidget(ContainerBase):
         nchild = self.widget.get_nth_page(idx)
         evbox = self.widget.get_tab_label(nchild)
         if tf:
-            evbox.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse('palegreen'))
+            GtkHelp.modify_bg(evbox, 'palegreen')
         else:
-            evbox.modify_bg(Gtk.StateType.NORMAL, None)
+            GtkHelp.modify_bg(evbox, None)
 
 
 class StackWidget(TabWidget):
@@ -1348,7 +1366,7 @@ class MDIWidget(ContainerBase):
         self.mode = 'mdi'
         self.true_mdi = True
 
-        # TODO: currently scrollbars are not working
+        # TODO: currently scrollbars are only partially working
         sw = Gtk.ScrolledWindow()
         sw.set_border_width(2)
         sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
@@ -1514,9 +1532,10 @@ class Splitter(ContainerBase):
         self.add_ref(child)
         child_w = child.get_widget()
 
-        # without a Frame it is difficult to see the divider
+        # without a Frame it can be difficult to see the divider
         frame_w = Gtk.Frame()
-        frame_w.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
+        #frame_w.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
+        frame_w.set_shadow_type(Gtk.ShadowType.NONE)
         frame_w.add(child_w)
 
         if len(self.children) == 1:
@@ -1633,9 +1652,12 @@ class Toolbar(ContainerBase):
             child = Button(text)
 
         if iconpath is not None:
-            wd, ht = 24, 24
             if iconsize is not None:
                 wd, ht = iconsize
+            else:
+                scale_f = _app.screen_res / 96.0
+                px = int(scale_f * 24)
+                wd, ht = px, px
             pixbuf = GtkHelp.pixbuf_new_from_file_at_size(iconpath, wd, ht)
             if pixbuf is not None:
                 image = Gtk.Image.new_from_pixbuf(pixbuf)
@@ -1753,10 +1775,7 @@ class Menu(ContainerBase):
     def popup(self, widget=None):
         menu = self.widget
         menu.show_all()
-        if six.PY2:
-            now = long(0)  # noqa
-        else:
-            now = int(0)
+        now = int(0)
         if menu.get_sensitive():
             menu.popup(None, None, None, None, 0, now)
 
@@ -1974,12 +1993,26 @@ class Application(Callback.Callbacks):
         self.wincnt = 0
 
         try:
-            screen = Gdk.screen_get_default()
-            self.screen_ht = screen.get_height()
-            self.screen_wd = screen.get_width()
-        except Exception:
+            display = Gdk.Display.get_default()
+            screen = display.get_default_screen()
+            window = screen.get_active_window()
+            monitor = screen.get_monitor_at_window(window)
+
+            g = screen.get_monitor_geometry(monitor)
+            self.screen_ht = g.height
+            self.screen_wd = g.width
+
+            self.screen_res = screen.get_resolution()
+
+            # hack for Gtk--scale fonts on HiDPI displays
+            scale = self.screen_res / 72.0
+            self.logger.debug("setting default font_scaling_factor={}".format(scale))
+            from ginga.fonts import font_asst
+            font_asst.default_scaling_factor = scale
+        except Exception as e:
             self.screen_wd = 1600
             self.screen_ht = 1200
+            self.screen_res = 96
         # self.logger.debug("screen dimensions %dx%d" % (
         #     self.screen_wd, self.screen_ht))
 
@@ -2043,6 +2076,9 @@ class Application(Callback.Callbacks):
         self.add_window(w)
         return w
 
+    def make_timer(self):
+        return GtkHelp.Timer()
+
     def mainloop(self):
         Gtk.main()
 
@@ -2136,7 +2172,9 @@ class DragPackage(object):
 
     def set_urls(self, urls):
         self._selection.set_uris(urls)
-        self._selection.set("text/plain", 0, '\n'.join(urls))
+
+    def set_text(self, text):
+        self._selection.set_text(text, len(text))
 
     def start_drag(self):
         pass
@@ -2217,12 +2255,20 @@ def make_widget(title, wtype):
 
 
 def hadjust(w, orientation):
+    """Ostensibly, a function to reduce the vertical footprint of a widget
+    that is normally used in a vertical stack (usually a Splitter), when it
+    is instead used in a horizontal orientation.
+    """
     if orientation != 'horizontal':
         return w
-    vbox = VBox()
-    vbox.add_widget(w)
-    vbox.add_widget(Label(''), stretch=1)
-    return vbox
+    # This currently does not seem to be needed for most plugins that are
+    # coded to flow either vertically or horizontally and, in fact, reduces
+    # the visual asthetic somewhat.
+    ## spl = Splitter(orientation='vertical')
+    ## spl.add_widget(w)
+    ## spl.add_widget(Label(''))
+    ## return spl
+    return w
 
 
 def build_info(captions, orientation='vertical'):
