@@ -359,8 +359,6 @@ class Desktop(Callback.Callbacks):
             else:
                 pack(widget)
 
-            # process_common_params(kind, widget, params)
-
             res = []
             if (kind in ('ws', 'mdi', 'grid', 'stack')) and (len(args) > 0):
                 # <-- Workspace specified a sub-layout.  We expect a list
@@ -394,9 +392,9 @@ class Desktop(Callback.Callbacks):
                     res.append(c)
 
                     # collect widths to set width of panes
-                    params = col[1]
-                    if 'width' in params:
-                        sizes.append(params['width'])
+                    s_params = col[1]
+                    if 'width' in s_params:
+                        sizes.append(s_params['width'])
 
                 sizes = params.get('sizes', sizes)
                 if len(sizes) == len(cols):
@@ -428,9 +426,9 @@ class Desktop(Callback.Callbacks):
                     res.append(r)
 
                     # collect heights to set height of panes
-                    params = row[1]
-                    if 'height' in params:
-                        sizes.append(params['height'])
+                    s_params = row[1]
+                    if 'height' in s_params:
+                        sizes.append(s_params['height'])
 
                 sizes = params.get('sizes', sizes)
                 if len(sizes) == len(rows):
@@ -693,18 +691,20 @@ class Workspace(Widgets.WidgetBase):
         self.nb = None
         self.group = group
         self.detachable = detachable
-        self.toolbar = None
         self.mdi_menu = None
         self.wstypes_c = ['Tabs', 'Grid', 'MDI', 'Stack']
         self.wstypes_l = ['tabs', 'grid', 'mdi', 'stack']
-        self.extdata = Bunch.Bunch()
+        self.extdata = Bunch.Bunch(toolbar=None)
+        self.toolbar = None
 
         if use_toolbar:
-            self.toolbar = Widgets.Toolbar(orientation='horizontal')
-            self.toolbar.set_border_width(0)
-            self.vbox.add_widget(self.toolbar, stretch=0)
+            toolbar = Widgets.Toolbar(orientation='horizontal')
+            toolbar.set_border_width(0)
+            self.vbox.add_widget(toolbar, stretch=0)
+            self.toolbar = toolbar
+            self.extdata.toolbar = toolbar
 
-            ws_menu = self.toolbar.add_menu("Workspace")
+            ws_menu = toolbar.add_menu("Workspace")
             item = ws_menu.add_name("Close")
             item.add_callback('activated', self._close_menuitem_cb)
 
@@ -731,6 +731,7 @@ class Workspace(Widgets.WidgetBase):
             item.add_callback('activated',
                               lambda *args: self.cascade_panes_cb())
             self.mdi_menu = mdi_menu
+            self.extdata.mdi_menu = mdi_menu
             self._update_mdi_menu()
 
         for name in ('page-switch', 'page-detach', 'page-close',
