@@ -7,6 +7,7 @@
 import glob
 import os
 import math
+import weakref
 
 import ginga.toolkit
 from ginga.util import iohelper
@@ -24,12 +25,17 @@ if toolkit == 'qt5':
 elif toolkit == 'qt4':
     os.environ['QT_API'] = 'pyqt'
 
+elif toolkit == 'pyside2':
+    os.environ['QT_API'] = 'pyside2'
+
 elif toolkit == 'pyside':
     os.environ['QT_API'] = 'pyside'
 
 have_pyqt4 = False
 have_pyqt5 = False
 have_pyside = False
+have_pyside2 = False
+qtpy_import_error = ""
 
 try:
     from qtpy import QtCore
@@ -43,14 +49,17 @@ try:
         pass
 
     # Let's see what qtpy configured for us...
-    from qtpy import PYQT4, PYQT5, PYSIDE
+    from qtpy import PYQT4, PYQT5, PYSIDE, PYSIDE2
     have_pyqt4 = PYQT4
     have_pyqt5 = PYQT5
     have_pyside = PYSIDE
+    have_pyside2 = PYSIDE2
 
     configured = True
 except ImportError as e:
-    pass
+    qtpy_import_error = "Error importing 'qtpy': {}".format(e)
+    # for debugging purposes, uncomment this to get full traceback
+    #raise e
 
 if have_pyqt5:
     ginga.toolkit.use('qt5')
@@ -58,11 +67,16 @@ if have_pyqt5:
 elif have_pyqt4:
     ginga.toolkit.use('qt4')
     os.environ['QT_API'] = 'pyqt'
+elif have_pyside2:
+    ginga.toolkit.use('pyside2')
+    os.environ['QT_API'] = 'pyside2'
 elif have_pyside:
     ginga.toolkit.use('pyside')
     os.environ['QT_API'] = 'pyside'
 else:
-    raise ImportError("Failed to configure qt4, qt5 or pyside. Is the 'qtpy' package installed?")
+    raise ImportError("Failed to configure qt4, qt5, pyside or pyside2. "
+                      "Is the 'qtpy' package installed? (%s)" % (
+                          qtpy_import_error))
 
 
 tabwidget_style = """
@@ -145,8 +159,11 @@ class HBox(QtGui.QWidget):
 
 class FileSelection(object):
     """Handle Load Image file dialog from File menu."""
-    def __init__(self, parent_w):
+    # TODO: deprecate the functionality when all_at_once == False
+    # and make the default to be True
+    def __init__(self, parent_w, all_at_once=False):
         self.parent = parent_w
+        self.all_at_once = all_at_once
         self.cb = None
 
     def popup(self, title, callfn, initialdir=None, filename=None):
@@ -177,6 +194,7 @@ class FileSelection(object):
         if ginga.toolkit.get_toolkit() == 'qt5':
             filenames = filenames[0]
 
+        all_paths = []
         for filename in filenames:
 
             # Special handling for wildcard or extension.
@@ -186,15 +204,21 @@ class FileSelection(object):
                 ext = iohelper.get_hdu_suffix(info.numhdu)
                 files = glob.glob(info.filepath)  # Expand wildcard
                 paths = ['{0}{1}'.format(f, ext) for f in files]
+                if self.all_at_once:
+                    all_paths.extend(paths)
+                else:
+                    for path in paths:
+                        self.cb(path)
 
-                # NOTE: Using drag-drop callback here might give QPainter
-                # warnings.
-                for path in paths:
-                    self.cb(path)
-
-            # Normal load
             else:
-                self.cb(filename)
+                # Normal load
+                if self.all_at_once:
+                    all_paths.append(filename)
+                else:
+                    self.cb(filename)
+
+        if self.all_at_once and len(all_paths) > 0:
+            self.cb(all_paths)
 
 
 class DirectorySelection(object):
@@ -239,6 +263,7 @@ class Timer(Callback.Callbacks):
 
         self.timer = QtCore.QTimer()
         self.timer.setSingleShot(True)
+        self.timer.setTimerType(QtCore.Qt.PreciseTimer)
         self.timer.timeout.connect(self._expired_cb)
 
         for name in ('expired', 'canceled'):
@@ -371,4 +396,20 @@ def load_font(font_name, font_file):
 
     return font_name
 
+
+# cache of QPainters for surfaces
+_painters = weakref.WeakKeyDictionary()
+
+
+def get_painter(surface):
+    if surface in _painters:
+        return _painters[surface]
+
+    painter = QPainter(surface)
+    painter.setRenderHint(QPainter.Antialiasing)
+    painter.setRenderHint(QPainter.TextAntialiasing)
+    _painters[surface] = painter
+    return painter
+
+
 # END
