@@ -5,7 +5,6 @@
 # Please see the file LICENSE.txt for details.
 #
 """This module handles the main reference viewer."""
-from __future__ import print_function
 
 # stdlib imports
 import glob
@@ -72,23 +71,26 @@ default_layout = ['seq', {},
 plugins = [
     # hidden plugins, started at program initialization
     Bunch(module='Operations', workspace='operations', start=True,
-          hidden=True, category='System', ptype='global'),
+          hidden=True, category='System', menu="Operations [G]",
+          ptype='global'),
     Bunch(module='Toolbar', workspace='toolbar', start=True,
-          hidden=True, category='System', ptype='global'),
+          hidden=True, category='System', menu="Toolbar [G]", ptype='global'),
     Bunch(module='Pan', workspace='uleft', start=True,
-          hidden=True, category='System', ptype='global'),
+          hidden=True, category='System', menu="Pan [G]", ptype='global'),
     Bunch(module='Info', tab='Synopsis', workspace='lleft', start=True,
-          hidden=True, category='System', ptype='global'),
+          hidden=True, category='System', menu="Info [G]", ptype='global'),
     Bunch(module='Thumbs', tab='Thumbs', workspace='right', start=True,
-          hidden=True, category='System', ptype='global'),
+          hidden=True, category='System', menu="Thumbs [G]", ptype='global'),
     Bunch(module='Contents', tab='Contents', workspace='right', start=True,
-          hidden=True, category='System', ptype='global'),
+          hidden=True, category='System', menu="Contents [G]", ptype='global'),
     Bunch(module='Colorbar', workspace='cbar', start=True,
-          hidden=True, category='System', ptype='global'),
+          hidden=True, category='System', menu="Colorbar [G]", ptype='global'),
     Bunch(module='Cursor', workspace='readout', start=True,
-          hidden=True, category='System', ptype='global'),
+          hidden=True, category='System', menu="Cursor [G]", ptype='global'),
     Bunch(module='Errors', tab='Errors', workspace='right', start=True,
-          hidden=True, category='System', ptype='global'),
+          hidden=True, category='System', menu="Errors [G]", ptype='global'),
+    Bunch(module='Downloads', tab='Downloads', workspace='right', start=False,
+          menu="Downloads [G]", category='Utils', ptype='global'),
 
     # optional, user-started plugins
     Bunch(module='Blink', tab='Blink Channels', workspace='right', start=False,
@@ -169,6 +171,7 @@ class ReferenceViewer(object):
     def __init__(self, layout=default_layout):
         self.plugins = []
         self.layout = layout
+        self.channels = ['Image']
 
     def add_plugin_spec(self, spec):
         self.plugins.append(spec)
@@ -214,62 +217,69 @@ class ReferenceViewer(object):
                 print("Error trying to instantiate external plugin using %s: %s" % (
                     str(method), str(e)))
 
-    def add_default_options(self, optprs):
+    def add_default_options(self, argprs):
         """
         Adds the default reference viewer startup options to an
-        OptionParser instance `optprs`.
+        ArgumentParser instance `argprs`.
         """
-        optprs.add_option("--bufsize", dest="bufsize", metavar="NUM",
-                          type="int", default=10,
-                          help="Buffer length to NUM")
-        optprs.add_option('-c', "--channels", dest="channels",
-                          help="Specify list of channels to create")
-        optprs.add_option("--debug", dest="debug", default=False,
-                          action="store_true",
-                          help="Enter the pdb debugger on main()")
-        optprs.add_option("--disable-plugins", dest="disable_plugins",
-                          metavar="NAMES",
-                          help="Specify plugins that should be disabled")
-        optprs.add_option("--display", dest="display", metavar="HOST:N",
-                          help="Use X display on HOST:N")
-        optprs.add_option("--fitspkg", dest="fitspkg", metavar="NAME",
-                          default=None,
-                          help="Prefer FITS I/O module NAME")
-        optprs.add_option("-g", "--geometry", dest="geometry",
-                          default=None, metavar="GEOM",
-                          help="X geometry for initial size and placement")
-        optprs.add_option("--modules", dest="modules", metavar="NAMES",
-                          help="Specify additional modules to load")
-        optprs.add_option("--norestore", dest="norestore", default=False,
-                          action="store_true",
-                          help="Don't restore the GUI from a saved layout")
-        optprs.add_option("--nosplash", dest="nosplash", default=False,
-                          action="store_true",
-                          help="Don't display the splash screen")
-        optprs.add_option("--numthreads", dest="numthreads", type="int",
-                          default=30, metavar="NUM",
-                          help="Start NUM threads in thread pool")
-        optprs.add_option("--opencv", dest="opencv", default=False,
-                          action="store_true",
-                          help="Use OpenCv acceleration")
-        optprs.add_option("--opencl", dest="opencl", default=False,
-                          action="store_true",
-                          help="Use OpenCL acceleration")
-        optprs.add_option("--plugins", dest="plugins", metavar="NAMES",
-                          help="Specify additional plugins to load")
-        optprs.add_option("--profile", dest="profile", action="store_true",
-                          default=False,
-                          help="Run the profiler on main()")
-        optprs.add_option("--sep", dest="separate_channels", default=False,
-                          action="store_true",
-                          help="Load files in separate channels")
-        optprs.add_option("-t", "--toolkit", dest="toolkit", metavar="NAME",
-                          default=None,
-                          help="Prefer GUI toolkit (gtk|qt)")
-        optprs.add_option("--wcspkg", dest="wcspkg", metavar="NAME",
-                          default=None,
-                          help="Prefer WCS module NAME")
-        log.addlogopts(optprs)
+        if hasattr(argprs, 'add_option'):
+            # older OptParse
+            add_argument = argprs.add_option
+        else:
+            # newer ArgParse
+            add_argument = argprs.add_argument
+
+        add_argument("--bufsize", dest="bufsize", metavar="NUM",
+                     type=int, default=10,
+                     help="Buffer length to NUM")
+        add_argument('-c', "--channels", dest="channels",
+                     help="Specify list of channels to create")
+        add_argument("--debug", dest="debug", default=False,
+                     action="store_true",
+                     help="Enter the pdb debugger on main()")
+        add_argument("--disable-plugins", dest="disable_plugins",
+                     metavar="NAMES",
+                     help="Specify plugins that should be disabled")
+        add_argument("--display", dest="display", metavar="HOST:N",
+                     help="Use X display on HOST:N")
+        add_argument("--fitspkg", dest="fitspkg", metavar="NAME",
+                     default=None,
+                     help="Prefer FITS I/O module NAME")
+        add_argument("-g", "--geometry", dest="geometry",
+                     default=None, metavar="GEOM",
+                     help="X geometry for initial size and placement")
+        add_argument("--modules", dest="modules", metavar="NAMES",
+                     help="Specify additional modules to load")
+        add_argument("--norestore", dest="norestore", default=False,
+                     action="store_true",
+                     help="Don't restore the GUI from a saved layout")
+        add_argument("--nosplash", dest="nosplash", default=False,
+                     action="store_true",
+                     help="Don't display the splash screen")
+        add_argument("--numthreads", dest="numthreads", type=int,
+                     default=30, metavar="NUM",
+                     help="Start NUM threads in thread pool")
+        add_argument("--opencv", dest="opencv", default=False,
+                     action="store_true",
+                     help="Use OpenCv acceleration")
+        add_argument("--opencl", dest="opencl", default=False,
+                     action="store_true",
+                     help="Use OpenCL acceleration")
+        add_argument("--plugins", dest="plugins", metavar="NAMES",
+                     help="Specify additional plugins to load")
+        add_argument("--profile", dest="profile", action="store_true",
+                     default=False,
+                     help="Run the profiler on main()")
+        add_argument("--sep", dest="separate_channels", default=False,
+                     action="store_true",
+                     help="Load files in separate channels")
+        add_argument("-t", "--toolkit", dest="toolkit", metavar="NAME",
+                     default=None,
+                     help="Prefer GUI toolkit (gtk|qt)")
+        add_argument("--wcspkg", dest="wcspkg", metavar="NAME",
+                     default=None,
+                     help="Prefer WCS module NAME")
+        log.addlogopts(argprs)
 
     def main(self, options, args):
         """
@@ -376,12 +386,6 @@ class ReferenceViewer(object):
                 logger.warning(
                     "failed to load matplotlib colormaps: %s" % (str(e)))
 
-        # user wants to set font scaling
-        font_scaling = settings.get('font_scaling_factor', None)
-        if font_scaling is not None:
-            from ginga.fonts import font_asst
-            font_asst.default_scaling_factor = font_scaling
-
         # Set a working RGB ICC profile if user has one
         working_profile = settings.get('icc_working_profile', None)
         rgb_cms.working_profile = working_profile
@@ -407,9 +411,14 @@ class ReferenceViewer(object):
             fitspkg = settings.get('FITSpkg', 'choose')
 
         try:
-            from ginga.util import io_fits
-            if wcspkg != 'choose':
+            from ginga.util import io_fits, loader
+            if fitspkg != 'choose':
                 assert io_fits.use(fitspkg) is True
+                # opener name is not necessarily the same
+                opener = loader.get_opener(io_fits.fitsLoaderClass.name)
+                # set this opener as the priority one
+                opener.priority = -99
+
         except Exception as e:
             logger.warning(
                 "failed to set FITS package preference: %s" % (str(e)))
@@ -447,6 +456,15 @@ class ReferenceViewer(object):
         ginga_shell = GingaShell(logger, thread_pool, mm, prefs,
                                  ev_quit=ev_quit)
 
+        # user wants to set font scaling.
+        # NOTE: this happens *after* creation of shell object, since
+        # Application object constructor will also set this
+        font_scaling = settings.get('font_scaling_factor', None)
+        if font_scaling is not None:
+            logger.debug("overriding font_scaling_factor to {}".format(font_scaling))
+            from ginga.fonts import font_asst
+            font_asst.default_scaling_factor = font_scaling
+
         layout_file = None
         if not options.norestore and settings.get('save_layout', False):
             layout_file = os.path.join(basedir, 'layout')
@@ -559,7 +577,7 @@ class ReferenceViewer(object):
         if options.channels is not None:
             channels = options.channels.split(',')
         else:
-            channels = settings.get('channels', [])
+            channels = settings.get('channels', self.channels)
             if not isinstance(channels, list):
                 channels = channels.split(',')
 
@@ -567,9 +585,15 @@ class ReferenceViewer(object):
             # should provide at least one default channel?
             channels = [settings.get('channel_prefix', "Image")]
 
-        for chname in channels:
-            ginga_shell.add_channel(chname)
-        ginga_shell.change_channel(channels[0])
+        # populate the initial channel lineup
+        for item in channels:
+            if isinstance(item, str):
+                chname, wsname = item, None
+            else:
+                chname, wsname = item
+            ginga_shell.add_channel(chname, workspace=wsname)
+
+        ginga_shell.change_channel(chname)
 
         # User configuration (custom star catalogs, etc.)
         if have_ginga_config:
@@ -594,16 +618,17 @@ class ReferenceViewer(object):
             logging.getLogger('py.warnings').addHandler(hdlr)
 
         # Display banner the first time run, unless suppressed
-        showBanner = True
+        show_banner = True
         try:
-            showBanner = settings.get('showBanner')
+            show_banner = settings.get('showBanner')
 
         except KeyError:
             # disable for subsequent runs
             settings.set(showBanner=False)
-            settings.save()
+            if not os.path.exists(settings.preffile):
+                settings.save()
 
-        if (not options.nosplash) and (len(args) == 0) and showBanner:
+        if (not options.nosplash) and (len(args) == 0) and show_banner:
             ginga_shell.banner(raiseTab=True)
 
         # Handle inputs like "*.fits[ext]" that sys cmd cannot auto expand.
@@ -622,25 +647,33 @@ class ReferenceViewer(object):
                 expanded_args.append(imgfile)
 
         # Assume remaining arguments are fits files and load them.
-        chname = None
-        for imgfile in expanded_args:
-            if options.separate_channels and (chname is not None):
-                channel = ginga_shell.add_channel_auto()
-            else:
-                channel = ginga_shell.get_channel_info()
-            chname = channel.name
-            ginga_shell.nongui_do(ginga_shell.load_file, imgfile,
-                                  chname=chname)
+        if not options.separate_channels:
+            chname = channels[0]
+            ginga_shell.gui_do(ginga_shell.open_uris, expanded_args,
+                               chname=chname)
+        else:
+            i = 0
+            num_channels = len(channels)
+            for imgfile in expanded_args:
+                if i < num_channels:
+                    chname = channels[i]
+                    i = i + 1
+                else:
+                    channel = ginga_shell.add_channel_auto()
+                    chname = channel.name
+                ginga_shell.gui_do(ginga_shell.open_uris, [imgfile],
+                                   chname=chname)
 
         try:
             try:
                 # if there is a network component, start it
                 if hasattr(ginga_shell, 'start'):
+                    logger.info("starting network interface...")
                     task = Task.FuncTask2(ginga_shell.start)
                     thread_pool.addTask(task)
 
                 # Main loop to handle GUI events
-                logger.info("Entering mainloop...")
+                logger.info("entering mainloop...")
                 ginga_shell.mainloop(timeout=0.001)
 
             except KeyboardInterrupt:
@@ -685,15 +718,14 @@ def reference_viewer(sys_argv):
     viewer.add_default_plugins()
     viewer.add_separately_distributed_plugins()
 
-    # Parse command line options with optparse module
-    from optparse import OptionParser
+    # Parse command line options with argparse module
+    from argparse import ArgumentParser
 
-    usage = "usage: %prog [options] cmd [args]"
-    optprs = OptionParser(usage=usage,
-                          version=('%%prog %s' % version.version))
-    viewer.add_default_options(optprs)
-
-    (options, args) = optprs.parse_args(sys_argv[1:])
+    argprs = ArgumentParser(description="Run the Ginga reference viewer.")
+    viewer.add_default_options(argprs)
+    argprs.add_argument('-V', '--version', action='version',
+                        version='%(prog)s {}'.format(version.version))
+    (options, args) = argprs.parse_known_args(sys_argv[1:])
 
     if options.display:
         os.environ['DISPLAY'] = options.display
@@ -709,7 +741,8 @@ def reference_viewer(sys_argv):
         import profile
 
         print(("%s profile:" % sys_argv[0]))
-        profile.run('viewer.main(options, args)')
+        profile.runctx('viewer.main(options, args)',
+                       dict(options=options, args=args, viewer=viewer), {})
 
     else:
         viewer.main(options, args)
