@@ -36,7 +36,7 @@ import threading
 import numpy as np
 
 from ginga.AstroImage import AstroImage
-from ginga.util import wcs, iqcalc, dp
+from ginga.util import wcs, iqcalc, dp, io_fits, mosaic
 from ginga import GingaPlugin
 from ginga.gw import Widgets
 
@@ -338,13 +338,11 @@ class Mosaic(GingaPlugin.LocalPlugin):
 
         time_intr1 = time.time()
 
-        loc = self.img_mosaic.mosaic_inline(images,
-                                            bg_ref=bg_ref,
-                                            trim_px=trim_px,
-                                            merge=merge,
-                                            allow_expand=allow_expand,
-                                            expand_pad_deg=expand_pad_deg,
-                                            suppress_callback=True)
+        loc = mosaic.mosaic_inline(self.img_mosaic, images,
+                                   bg_ref=bg_ref, trim_px=trim_px,
+                                   merge=merge, allow_expand=allow_expand,
+                                   expand_pad_deg=expand_pad_deg,
+                                   suppress_callback=True)
 
         # Add description for ChangeHistory
         info = dict(time_modified=datetime.utcnow(),
@@ -445,15 +443,14 @@ class Mosaic(GingaPlugin.LocalPlugin):
                 mosaic_hdus = self.settings.get('mosaic_hdus', False)
                 if mosaic_hdus:
                     self.logger.debug("loading hdus")
-                    opener = self.fv.fits_opener.get_factory()
+                    opener = io_fits.get_fitsloader(logger=self.logger)
                     # User wants us to mosaic HDUs
-                    # TODO: do this in a different thread?
                     opener.open_file(url, memmap=False)
                     try:
                         for i in range(len(opener)):
                             self.logger.debug("ingesting hdu #%d" % (i))
                             try:
-                                image = opener.get_hdu(i)
+                                image = opener.load_idx(i)
 
                             except Exception as e:
                                 self.logger.error(
@@ -465,6 +462,15 @@ class Mosaic(GingaPlugin.LocalPlugin):
                                     "HDU #%d is not an image; skipping..." % (i))
                                 continue
 
+                            data = image.get_data()
+                            if data is None:
+                                # skip blank data
+                                continue
+
+                            if image.ndim != 2:
+                                # skip images without 2 dimensions
+                                continue
+
                             image.set(name='hdu%d' % (i))
 
                             image = self.preprocess(image)
