@@ -1,5 +1,7 @@
+#
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
+#
 """
 A plugin to navigate HDUs in a FITS file or planes in a 3D cube or
 higher dimension dataset.
@@ -47,7 +49,7 @@ from contextlib import contextmanager
 from ginga.gw import Widgets
 from ginga.misc import Future
 from ginga import GingaPlugin
-from ginga.util.iohelper import get_hdu_suffix
+from ginga.util import iohelper
 from ginga.util.videosink import VideoSink
 
 import numpy as np
@@ -64,6 +66,7 @@ class MultiDim(GingaPlugin.LocalPlugin):
     def __init__(self, fv, fitsimage):
         # superclass defines some variables for us, like logger
         super(MultiDim, self).__init__(fv, fitsimage)
+        self._toc_fmt = "%(index)4d %(name)-12.12s (%(extver)3d) %(htype)-12.12s %(dtype)-8.8s"
 
         self.curhdu = 0
         self.naxispath = []
@@ -88,7 +91,8 @@ class MultiDim(GingaPlugin.LocalPlugin):
         # Load plugin preferences
         prefs = self.fv.get_preferences()
         self.settings = prefs.create_category('plugin_MultiDim')
-        self.settings.set_defaults(auto_start_naxis=False)
+        self.settings.add_defaults(sort_keys=['index'],
+                                   sort_reverse=False)
         self.settings.load(onError='silent')
 
         self.gui_up = False
@@ -215,14 +219,13 @@ class MultiDim(GingaPlugin.LocalPlugin):
         self.gui_up = True
 
     def set_hdu_cb(self, w, val):
-        # idx = int(val)
-        idx = w.get_index()
-        idx = max(0, idx)
+        # Get actual HDU index, which might be different from combobox order.
+        toc_ent = w.get_alpha(val)
+        idx = max(0, int(toc_ent[:4]))
         try:
             self.set_hdu(idx)
-
         except Exception as e:
-            self.logger.error("Error loading HDU #%d: %s" % (
+            self.logger.error("Error loading HDU #{}: {}".format(
                 idx + 1, str(e)))
 
     def set_naxis_cb(self, widget, idx, n):
@@ -231,12 +234,13 @@ class MultiDim(GingaPlugin.LocalPlugin):
 
     def build_naxis(self, dims, image):
         self.naxispath = list(image.naxispath)
+        ndims = len(dims)
 
         # build a vbox of NAXIS controls
         captions = [("NAXIS1:", 'label', 'NAXIS1', 'llabel'),
                     ("NAXIS2:", 'label', 'NAXIS2', 'llabel')]
 
-        for n in range(2, len(dims)):
+        for n in range(2, ndims):
             key = 'naxis%d' % (n + 1)
             title = key.upper()
             maxn = int(dims[n])
@@ -254,9 +258,9 @@ class MultiDim(GingaPlugin.LocalPlugin):
 
         hbox = Widgets.HBox()
 
-        if len(dims) > 3:  # only add radiobuttons if we have more than 3 dim
+        if ndims > 3:  # only add radiobuttons if we have more than 3 dim
             group = None
-            for i in range(2, len(dims)):
+            for i in range(2, ndims):
                 title = 'AXIS%d' % (i + 1)
                 btn = Widgets.RadioButton(title, group=group)
                 if group is None:
@@ -271,7 +275,7 @@ class MultiDim(GingaPlugin.LocalPlugin):
         vbox.add_widget(w)
         vbox.add_widget(hbox)
 
-        for n in range(0, len(dims)):
+        for n in range(0, ndims):
             key = 'naxis%d' % (n + 1)
             lbl = b[key]
             maxn = int(dims[n])
@@ -303,10 +307,10 @@ class MultiDim(GingaPlugin.LocalPlugin):
 
         # for storing play_idx for each dim of image. used for going back to
         # the idx where you left off.
-        self.play_indices = ([0 for i in range(len(dims) - 2)] if len(dims) > 3
+        self.play_indices = ([0 for i in range(ndims - 2)] if ndims > 3
                              else None)
 
-        if len(dims) > 3:
+        if ndims > 3:
 
             # dims only exists in here, hence this function exists here
             def play_axis_change_func_creator(n):
@@ -318,7 +322,7 @@ class MultiDim(GingaPlugin.LocalPlugin):
                     self.play_axis = n
                     self.logger.debug("play_axis changed to %d" % n)
 
-                    if self.play_axis < len(dims):
+                    if self.play_axis < ndims:
                         self.play_max = dims[self.play_axis] - 1
 
                     self.play_idx = self.play_indices[n - 2]
@@ -332,16 +336,16 @@ class MultiDim(GingaPlugin.LocalPlugin):
 
                 return check_if_we_need_change
 
-            for n in range(2, len(dims)):
+            for n in range(2, ndims):
                 key = 'axis%d' % (n + 1)
                 self.w[key].add_callback(
                     'activated', play_axis_change_func_creator(n))
                 if n == 2:
                     self.w[key].set_state(True)
 
-        is_dc = len(dims) > 2
+        is_dc = ndims > 2
         self.play_axis = 2
-        if self.play_axis < len(dims):
+        if self.play_axis < ndims:
             self.play_max = dims[self.play_axis] - 1
         if is_dc:
             self.play_idx = self.naxispath[self.play_axis - 2]
@@ -394,14 +398,16 @@ class MultiDim(GingaPlugin.LocalPlugin):
         self.fv.show_status("")
 
     def set_hdu(self, idx):
-        self.logger.debug("Loading fits hdu #%d" % (idx))
+        self.logger.debug("Loading index #%d" % (idx))
 
         # determine canonical index of this HDU
-        info = self.file_obj.hdu_info[idx]
+        info_dct = self.file_obj.get_directory()
+        info = self.file_obj.get_info_idx(idx)
+
         aidx = (info.name, info.extver)
-        if aidx not in self.file_obj.hdu_db:
+        if aidx not in info_dct:
             aidx = idx
-        sfx = get_hdu_suffix(aidx)
+        sfx = iohelper.get_hdu_suffix(aidx)
 
         # See if this HDU is still in the channel's datasrc
         imname = self.name_pfx + sfx
@@ -414,27 +420,27 @@ class MultiDim(GingaPlugin.LocalPlugin):
             return
 
         # Nope, we'll have to load it
-        self.logger.debug("HDU %d not in memory; refreshing from file" % (idx))
-        try:
-            self.curhdu = idx
-            info = self.file_obj.hdu_info[idx]
-            image = self.file_obj.get_hdu(idx)
+        self.logger.debug("Index %d not in memory; refreshing from file" % (idx))
 
-            # create a future for reconstituting this HDU
-            future = Future.Future()
-            future.freeze(self.fv.load_image, self.img_path, idx=aidx)
-            image.set(path=self.img_path, idx=aidx, name=imname,
-                      image_future=future)
+        def _load_idx(image):
+            try:
+                # create a future for reconstituting this HDU
+                future = Future.Future()
+                future.freeze(self.fv.load_image, self.img_path, idx=aidx)
+                image.set(path=self.img_path, idx=aidx, name=imname,
+                          image_future=future)
 
-            self.fv.add_image(imname, image, chname=chname)
+                self.fv.add_image(imname, image, chname=chname)
+                self.curhdu = idx
+                self.logger.debug("HDU #%d loaded." % (idx))
 
-            self.logger.debug("HDU #%d loaded." % (idx))
+            except Exception as e:
+                errmsg = "Error loading FITS HDU #%d: %s" % (
+                    idx, str(e))
+                self.logger.error(errmsg)
+                self.fv.show_error(errmsg, raisetab=True)
 
-        except Exception as e:
-            errmsg = "Error loading FITS HDU #%d: %s" % (
-                idx, str(e))
-            self.logger.error(errmsg)
-            self.fv.show_error(errmsg, raisetab=True)
+        self.file_obj.load_idx_cont(idx, _load_idx)
 
     def set_naxis(self, idx, n):
         """Change the slice shown in the channel viewer.
@@ -507,7 +513,8 @@ class MultiDim(GingaPlugin.LocalPlugin):
         deadline = time_start + self.play_int_sec
 
         # calculate fps
-        fps = 1.0 / (time_start - self.play_last_time)
+        dt = abs(time_start - self.play_last_time)
+        fps = 1.0 / dt if not np.isclose(dt, 0) else 1.0
         self.play_last_time = time_start
         if int(fps) != int(self.play_fps):
             self.play_fps = fps
@@ -561,15 +568,22 @@ class MultiDim(GingaPlugin.LocalPlugin):
         # clear old TOC
         w.clear()
 
-        for idx, d in enumerate(hdu_info):
-            toc_ent = "%(index)4d %(name)-12.12s (%(extver)3d) %(htype)-12.12s %(dtype)-8.8s" % d  # noqa
+        # User sort settings
+        sort_keys = self.settings.get('sort_keys', ['index'])
+        sort_reverse = self.settings.get('sort_reverse', False)
+        sorted_hdu_info = sorted(hdu_info,
+                                 key=lambda x: [x[key] for key in sort_keys],
+                                 reverse=sort_reverse)
+
+        for idx, d in enumerate(sorted_hdu_info):
+            toc_ent = self._toc_fmt % d
             w.append_text(toc_ent)
 
-        idx = w.get_index()
-        if idx < 0:
-            idx = 0
-        if idx >= len(hdu_info):
-            idx = len(hdu_info) - 1
+        # idx = w.get_index()
+        # if idx < 0:
+        #     idx = 0
+        # if idx >= len(hdu_info):
+        #     idx = len(hdu_info) - 1
         # w.set_index(idx)
 
     def redo(self):
@@ -584,6 +598,12 @@ class MultiDim(GingaPlugin.LocalPlugin):
                 "Cannot open image: no value for metadata key 'path'")
             return
 
+        # TODO: How to properly reset GUI components?
+        #       They are still showing info from prev FITS.
+        # No-op for ASDF
+        if path.endswith('.asdf'):
+            return True
+
         if path != self.img_path:
             # <-- New file is being looked at
             self.img_path = path
@@ -595,19 +615,25 @@ class MultiDim(GingaPlugin.LocalPlugin):
                 except Exception:
                     pass
 
-            self.file_obj = self.fv.fits_opener.get_factory()
+            self.file_obj = image.io
+
             # TODO: specify 'readonly' somehow?
             self.file_obj.open_file(path)
+            hdu_dct = self.file_obj.get_directory()
 
             upper = len(self.file_obj) - 1
-            self.prep_hdu_menu(self.w.hdu, self.file_obj.hdu_info)
+            # NOTE: make a set of values, because some values will be in
+            # multiple times if known by several indexes
+            self.prep_hdu_menu(self.w.hdu, list(set(hdu_dct.values())))
             self.num_hdu = upper
             self.logger.debug("there are %d hdus" % (upper + 1))
             self.w.numhdu.set_text("%d" % (upper + 1))
 
             self.w.hdu.set_enabled(len(self.file_obj) > 0)
+        else:
+            hdu_dct = self.file_obj.get_directory()
 
-        name = image.get('name', self.fv.name_image_from_path(path))
+        name = image.get('name', iohelper.name_image_from_path(path))
         idx = image.get('idx', None)
         # remove index designation from root of name, if any
         match = re.match(r'^(.+)\[(.+)\]$', name)
@@ -618,11 +644,11 @@ class MultiDim(GingaPlugin.LocalPlugin):
         htype = None
         if idx is not None:
             # set the HDU in the drop down if known
-            info = self.file_obj.hdu_db.get(idx, None)
+            info = hdu_dct.get(idx, None)
             if info is not None:
                 htype = info.htype.lower()
-                index = info.index
-                self.w.hdu.set_index(index)
+                toc_ent = self._toc_fmt % info
+                self.w.hdu.show_text(toc_ent)
 
         # rebuild the NAXIS controls, if necessary
         # No two images in the same channel can have the same name.
@@ -633,13 +659,14 @@ class MultiDim(GingaPlugin.LocalPlugin):
             data = image.get_data()
             if data is None:
                 # <- empty data part to this HDU
-                self.logger.warning("Empty data part in HDU #%d" % (idx))
+                self.logger.warning("Empty data part in HDU %s" % (str(idx)))
 
             elif htype in ('bintablehdu', 'tablehdu',):
                 pass
 
-            elif htype not in ('imagehdu', 'primaryhdu'):
-                self.logger.warning("HDU #%d is not an image" % (idx))
+            elif htype not in ('imagehdu', 'primaryhdu', 'compimagehdu'):
+                self.logger.warning("HDU %s is not an image (%s)" % (
+                    str(idx), htype))
 
             else:
                 mddata = image.get_mddata()
