@@ -11,7 +11,7 @@ An instance can be opened for each channel.
 **Usage**
 
 The ``Pick`` plugin is used to perform quick astronomical data quality analysis
-on stellar objects.  It locates stellar candidates within a drawn rectangle
+on stellar objects.  It locates stellar candidates within a drawn box
 and picks the most likely candidate based on a set of search settings.
 The Full Width Half Max (FWHM) is reported on the candidate object, as
 well as its size based on the plate scale of the detector.  Rough
@@ -19,7 +19,7 @@ measurement of background, sky level and brightness is also done.
 
 **Defining the pick area**
 
-The default pick area is defined as a rectangle of approximately 30x30
+The default pick area is defined as a box of approximately 30x30
 pixels that encloses the search area.
 
 The move/draw/edit selector at the bottom of the plugin is used to
@@ -37,7 +37,7 @@ determine what operation is being done to the pick area:
   If there is no existing area, a default one will be created.
 * If "draw" is selected, then you can draw a shape with the cursor
   to enclose and define a new pick area.  The default shape is a
-  rectangle, but other shapes can be selected in the "Settings" tab.
+  box, but other shapes can be selected in the "Settings" tab.
 * If "edit" is selected, then you can edit the pick area by dragging its
   control points, or moving it by dragging in the bounding box.
 
@@ -86,8 +86,8 @@ pan position will be set to the found center.
 The "Contour" tab will show a contour plot.
 This is a contour plot of the area immediately surrounding the
 candidate, and not usually encompassing the entire region of the pick
-area.  You can use the vertical slider to the right of the plot to
-increase or decrease the area of the contour plot.
+area.  You can use the scroll wheel to zoom the plot and a click of the
+scroll wheel (mouse button 2) to set the pan position in the plot.
 
 .. figure:: figures/pick-fwhm.png
    :width: 400px
@@ -133,7 +133,7 @@ this plot is not updated.
    "Readout" tab of ``Pick`` area.
 
 The "Readout" tab will be populated with a summary of the measurements.
-There are two buttons and two check boxes in this tab:
+There are two buttons and three check boxes in this tab:
 
 * The "Default Region" button restores the pick region to the default
   shape and size.
@@ -143,6 +143,8 @@ There are two buttons and two check boxes in this tab:
   This affects the behavior of the pick region as described above.
 * The "From Peak" check box changes the behavior of "Quick Mode" slightly
   as described above.
+* If "Center on pick" is checked, the shape will be recentered on the
+  located center, if found (i.e. the shape "tracks" the pick)
 
 .. figure:: figures/pick-controls.png
    :width: 400px
@@ -288,7 +290,6 @@ from ginga.gw import Widgets, Viewers
 from ginga.misc import Bunch
 from ginga.util import iqcalc, wcs, contour
 from ginga import GingaPlugin, colors, cmap, trcalc
-from ginga.util.six.moves import map
 
 try:
     from ginga.gw import Plot
@@ -324,9 +325,9 @@ class Pick(GingaPlugin.LocalPlugin):
         self.contour_interp_methods = trcalc.interpolation_methods
 
         # types of pick shapes that can be drawn
-        self.drawtypes = ['rectangle', 'box', 'squarebox',
+        self.drawtypes = ['box', 'squarebox', 'rectangle',
                           'circle', 'ellipse',
-                          'freepolygon', 'polygon', 'triangle',
+                          'freepolygon', 'polygon',
                           ]
 
         # get Pick preferences
@@ -349,6 +350,8 @@ class Pick(GingaPlugin.LocalPlugin):
         self.ev_intr = threading.Event()
         self.tmr_quick = fv.get_timer()
         self.tmr_quick.set_callback('expired', self.quick_timer_cb)
+        self._wd, self._ht = 400, 300
+        self._split_sizes = [self._ht, self._ht]
 
         self.last_rpt = []
         self.rpt_dict = OrderedDict({})
@@ -395,11 +398,12 @@ class Pick(GingaPlugin.LocalPlugin):
             self.canvas.add(self.cross)
 
         self.have_mpl = have_mpl
+        self.gui_up = False
 
     def sync_preferences(self):
         # Load various preferences
         self.pickcolor = self.settings.get('color_pick', 'green')
-        self.pickshape = self.settings.get('shape_pick', 'rectangle')
+        self.pickshape = self.settings.get('shape_pick', 'box')
         if self.pickshape not in self.drawtypes:
             self.pickshape = 'box'
         self.candidate_color = self.settings.get('color_candidate', 'orange')
@@ -426,6 +430,7 @@ class Pick(GingaPlugin.LocalPlugin):
         self.center_alg = self.settings.get('calc_center_alg', 'fwhm')
         self.fwhm_algs = ['gaussian', 'moffat']
         self.fwhm_alg = self.settings.get('calc_fwhm_alg', 'gaussian')
+        self.center_on_pick = self.settings.get('center_on_pick', False)
 
         # For controls
         self.delta_bg = self.settings.get('delta_bg', 0.0)
@@ -464,6 +469,7 @@ class Pick(GingaPlugin.LocalPlugin):
         box.set_spacing(2)
 
         paned = Widgets.Splitter(orientation=orientation)
+        self.w.splitter = paned
 
         nb = Widgets.TabWidget(tabpos='bottom')
         self.w.nb1 = nb
@@ -472,7 +478,7 @@ class Pick(GingaPlugin.LocalPlugin):
         cm, im = self.fv.cm, self.fv.im
 
         di = Viewers.CanvasView(logger=self.logger)
-        width, height = 400, 300
+        width, height = self._wd, self._ht
         di.set_desired_size(width, height)
         di.enable_autozoom('off')
         di.enable_autocuts('off')
@@ -563,7 +569,6 @@ class Pick(GingaPlugin.LocalPlugin):
         if have_mpl:
             if not contour.have_skimage:
                 # Contour plot
-                hbox = Widgets.HBox()
                 self.contour_plot = plots.ContourPlot(
                     logger=self.logger, width=width, height=height)
                 if plots.MPL_GE_2_0:
@@ -571,33 +576,17 @@ class Pick(GingaPlugin.LocalPlugin):
                 else:
                     kwargs = {'axisbg': 'black'}
                 self.contour_plot.add_axis(**kwargs)
+                self.contour_plot.enable(pan=True, zoom=True)
                 pw = Plot.PlotWidget(self.contour_plot)
                 pw.resize(width, height)
-                hbox.add_widget(pw, stretch=1)
+                self.contour_plot.connect_ui()
 
                 self.contour_interp_methods = ('bilinear', 'nearest', 'bicubic')
                 if self.contour_interpolation not in self.contour_interp_methods:
                     self.contour_interpolation = 'nearest'
                 self.contour_plot.interpolation = self.contour_interpolation
 
-                # calc contour zoom setting
-                max_z = 100
-                zv = int(np.sqrt(self.dx**2 + self.dy**2) * 0.15)
-                zv = max(1, min(zv, 100))
-                self.contour_plot.plot_zoomlevel = zv
-
-                zoom = Widgets.Slider(orientation='vertical', track=True)
-                zoom.set_limits(1, max_z, incr_value=1)
-                zoom.set_value(zv)
-                self.w.zoom_sldr = zoom
-
-                def zoom_contour_cb(w, val):
-                    zl = val / 10.0
-                    self.contour_plot.plot_zoom(zl)
-
-                zoom.add_callback('value-changed', zoom_contour_cb)
-                hbox.add_widget(zoom, stretch=0)
-                nb.add_widget(hbox, title="Contour")
+                nb.add_widget(pw, title="Contour")
 
             # FWHM gaussians plot
             self.fwhm_plot = plots.FWHMPlot(logger=self.logger,
@@ -649,7 +638,8 @@ class Pick(GingaPlugin.LocalPlugin):
                      'Star Size:', 'label', 'Star Size', 'llabel'),
                     ('Sample Area:', 'label', 'Sample Area', 'llabel',
                      'Default Region', 'button', 'Pan to pick', 'button'),
-                    ('Quick Mode', 'checkbutton', 'From Peak', 'checkbutton'),
+                    ('Quick Mode', 'checkbutton', 'From Peak', 'checkbutton',
+                     'Center on pick', 'checkbutton'),
                     )
 
         w, b = Widgets.build_info(captions, orientation=orientation)
@@ -679,6 +669,10 @@ class Pick(GingaPlugin.LocalPlugin):
         ## b.drag_only.set_tooltip("In quick mode, require cursor press or follow cursor")
         ## b.drag_only.add_callback('activated', self.drag_only_cb)
         ## b.drag_only.set_state(self.drag_only)
+        b.center_on_pick.add_callback('activated', self.center_on_pick_cb)
+        b.center_on_pick.set_state(self.center_on_pick)
+        b.center_on_pick.set_tooltip("When peak is found, center shape\n"
+                                     "on peak.")
 
         vbox1 = Widgets.VBox()
         vbox1.add_widget(w, stretch=0)
@@ -1052,8 +1046,8 @@ class Pick(GingaPlugin.LocalPlugin):
 
         box.add_widget(fr, stretch=5)
         paned.add_widget(sw)
-        # hack to set a reasonable starting position for the splitter
-        paned.set_sizes([height, height])
+        paned.set_sizes(self._split_sizes)
+
         vtop.add_widget(paned, stretch=5)
 
         mode = self.canvas.get_draw_mode()
@@ -1099,6 +1093,7 @@ class Pick(GingaPlugin.LocalPlugin):
         vtop.add_widget(btns, stretch=0)
 
         container.add_widget(vtop, stretch=5)
+        self.gui_up = True
 
     def record_cb(self, w, tf):
         self.do_record = tf
@@ -1194,10 +1189,6 @@ class Pick(GingaPlugin.LocalPlugin):
             if self.contour_plot is not None:
                 self.contour_plot.plot_contours_data(
                     x, y, data, num_contours=self.num_contours)
-                # make zoom control match contour zoom
-                zl = self.contour_plot.plot_zoomlevel
-                zv = int(round(max(1, min(zl * 10, 100))))
-                self.w.zoom_sldr.set_value(zv)
 
         except Exception as e:
             self.logger.error("Error making contour plot: %s" % (
@@ -1264,9 +1255,11 @@ class Pick(GingaPlugin.LocalPlugin):
         self.modes_off()
 
         self.canvas.ui_set_active(True)
-        self.fv.show_status("Draw a rectangle with the right mouse button")
+        self.fv.show_status("Draw a shape with the right mouse button")
 
     def stop(self):
+        self.gui_up = False
+        self._split_sizes = self.w.splitter.get_sizes()
         # Delete previous peak marks
         objs = self.canvas.get_objects_by_tag_pfx('peak')
         self.canvas.delete_objects(objs)
@@ -1308,7 +1301,7 @@ class Pick(GingaPlugin.LocalPlugin):
         ctr_x, ctr_y = shape.get_center_pt()
         point.x, point.y = ctr_x, ctr_y
         x1, y1, x2, y2 = shape.get_llur()
-        text.x, text.y = x1, y2 + 4
+        text.x, text.y = x1, y2
 
         try:
             image = self.fitsimage.get_image()
@@ -1499,7 +1492,7 @@ class Pick(GingaPlugin.LocalPlugin):
             text = pickobj.objects[2]
             text.text = self._textlabel
             _x1, _y1, x2, y2 = shape_obj.get_llur()
-            text.x, text.y = x1, y2 + 4
+            text.x, text.y = x1, y2
 
             if msg is not None:
                 raise Exception(msg)
@@ -1523,6 +1516,12 @@ class Pick(GingaPlugin.LocalPlugin):
             point.x, point.y = obj_x, obj_y
             text.color = 'cyan'
 
+            if self.center_on_pick:
+                shape_obj.move_to_pt((obj_x, obj_y))
+                # reposition label above moved shape
+                _x1, _y1, x2, y2 = shape_obj.get_llur()
+                text.x, text.y = _x1, y2
+
             # Make report
             self.last_rpt = reports
             if self.do_record:
@@ -1736,7 +1735,7 @@ class Pick(GingaPlugin.LocalPlugin):
     def reset_region(self):
         self.dx = region_default_width
         self.dy = region_default_height
-        self.set_drawtype('rectangle')
+        self.set_drawtype('box')
 
         obj = self.pick_obj
         if obj.kind != 'compound':
@@ -1750,10 +1749,9 @@ class Pick(GingaPlugin.LocalPlugin):
         x2, y2 = data_x + rd_x, data_y + rd_y
 
         # replace shape
-        # TODO: makes sense to change this to 'box'
-        Rect = self.dc.Rectangle
-        tag = self.canvas.add(Rect(x1, y1, x2, y2,
-                                   color=self.pickcolor))
+        Box = self.dc.Box
+        tag = self.canvas.add(Box(data_x, data_y, self.dx // 2, self.dy // 2,
+                                  color=self.pickcolor))
 
         self.draw_cb(self.canvas, tag)
 
@@ -1769,7 +1767,7 @@ class Pick(GingaPlugin.LocalPlugin):
         obj.color = self.pickcolor
         args = [obj,
                 self.dc.Point(x, y, 10, color='red'),
-                self.dc.Text(x1, y2 + 4, '{0}: calc'.format(self._textlabel),
+                self.dc.Text(x1, y2, '{0}: calc'.format(self._textlabel),
                              color=self.pickcolor)
                 ]
 
@@ -1962,15 +1960,14 @@ class Pick(GingaPlugin.LocalPlugin):
 
         else:
             # No object yet? Add a default one.
-            self.set_drawtype('rectangle')
+            self.set_drawtype('box')
             rd_x, rd_y = self.dx // 2, self.dy // 2
             x1, y1 = data_x - rd_x, data_y - rd_y
             x2, y2 = data_x + rd_x, data_y + rd_y
 
-            # TODO: makes sense to change this to 'box'
-            Rect = self.canvas.get_draw_class('rectangle')
-            tag = self.canvas.add(Rect(x1, y1, x2, y2,
-                                       color=self.pickcolor))
+            Box = self.canvas.get_draw_class('box')
+            tag = self.canvas.add(Box(data_x, data_y, rd_x, rd_y,
+                                      color=self.pickcolor))
 
             self.draw_cb(self.canvas, tag)
 
@@ -2053,6 +2050,10 @@ class Pick(GingaPlugin.LocalPlugin):
         self.from_peak = tf
         return True
 
+    def center_on_pick_cb(self, w, tf):
+        self.center_on_pick = tf
+        return True
+
     def drag_only_cb(self, w, tf):
         self.drag_only = tf
         return True
