@@ -23,6 +23,10 @@ image.
 The "Auto Scroll" checkbox, if checked, will cause the ``Thumbs`` pan to
 scroll to the active image.
 
+This plugin is not usually configured to be closeable, but the user can
+make it so by setting the "closeable" setting to True in the configuration
+file--then Close and Help buttons will be added to the bottom of the UI.
+
 """
 import os
 import math
@@ -30,12 +34,13 @@ import time
 import threading
 
 from ginga import GingaPlugin
-from ginga import RGBImage, BaseImage
+from ginga import RGBImage
 from ginga.misc import Bunch
 from ginga.util import iohelper
 from ginga.gw import Widgets, Viewers
 from ginga.util.paths import icondir
 from ginga.pilw.ImageViewPil import CanvasView
+from ginga.util import io_rgb
 
 __all__ = ['Thumbs']
 
@@ -59,6 +64,8 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         self._displayed_thumb_dict = {}
         tt_keywords = ['OBJECT', 'FRAMEID', 'UT', 'DATE-OBS']
 
+        spec = self.fv.get_plugin_spec(str(self))
+
         prefs = self.fv.get_preferences()
         self.settings = prefs.create_category('plugin_Thumbs')
         self.settings.add_defaults(cache_thumbs=False,
@@ -79,6 +86,7 @@ class Thumbs(GingaPlugin.GlobalPlugin):
                                    label_bg_color='lightgreen',
                                    autoload_visible_thumbs=True,
                                    autoload_interval=1.0,
+                                   closeable=not spec.get('hidden', False),
                                    transfer_attrs=['transforms',
                                                    'cutlevels', 'rgbmap'])
         self.settings.load(onError='silent')
@@ -89,6 +97,9 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         self.thumb_vsep = self.settings.get('thumb_vsep', 15)
         self.transfer_attrs = self.settings.get('transfer_attrs', [])
 
+        # used to open thumbnails on disk
+        self.rgb_opener = io_rgb.RGBFileHandler(self.logger)
+
         # Build our thumb generator
         tg = CanvasView(logger=self.logger)
         tg.configure_surface(self.thumb_width, self.thumb_width)
@@ -138,7 +149,7 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         vbox.set_border_width(4)
         vbox.set_spacing(2)
 
-        # construct an interaactive viewer to view and scroll
+        # construct an interactive viewer to view and scroll
         # the thumbs pane
         self.c_view = Viewers.CanvasView(logger=self.logger)
         c_v = self.c_view
@@ -182,7 +193,8 @@ class Thumbs(GingaPlugin.GlobalPlugin):
 
         vbox.add_widget(iw, stretch=1)
 
-        captions = (('Auto scroll', 'checkbutton', 'Clear', 'button'),)
+        captions = (('Auto scroll', 'checkbutton', 'Clear', 'button'),
+                    )
         w, b = Widgets.build_info(captions)
         self.w.update(b)
 
@@ -194,6 +206,20 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         b.auto_scroll.set_state(auto_scroll)
         vbox.add_widget(w, stretch=0)
 
+        if self.settings.get('closeable', False):
+            btns = Widgets.HBox()
+            btns.set_border_width(4)
+            btns.set_spacing(4)
+
+            btn = Widgets.Button("Close")
+            btn.add_callback('activated', lambda w: self.close())
+            btns.add_widget(btn)
+            btn = Widgets.Button("Help")
+            btn.add_callback('activated', lambda w: self.help())
+            btns.add_widget(btn, stretch=0)
+            btns.add_widget(Widgets.Label(''), stretch=1)
+            vbox.add_widget(btns, stretch=0)
+
         container.add_widget(vbox, stretch=1)
 
         self.gui_up = True
@@ -202,7 +228,25 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         """Punt drag-drops to the ginga shell.
         """
         channel = self.fv.get_current_channel()
-        self.fv.dragdrop(channel.viewer, urls)
+        if channel is None:
+            return
+        self.fv.open_uris(urls, chname=channel.name, bulk_add=True)
+        return True
+
+    def start(self):
+        self.add_visible_thumbs()
+
+    def stop(self):
+        self.gui_up = False
+
+    def close(self):
+        # clear current thumbs
+        canvas = self.c_view.get_canvas()
+        canvas.delete_all_objects()
+        self._displayed_thumb_dict = dict()
+
+        self.fv.stop_global_plugin(str(self))
+        self.c_view = None
         return True
 
     def _get_thumb_key(self, chname, image):
@@ -466,7 +510,8 @@ class Thumbs(GingaPlugin.GlobalPlugin):
                     namelbl = self.thumb_dict[thumbkey].get('namelbl')
                     namelbl.color = bg
 
-        self.c_view.redraw(whence=3)
+        if self.gui_up:
+            self.c_view.redraw(whence=3)
 
     def redo(self, channel, image):
         """This method is called when an image is set in a channel."""
@@ -513,7 +558,7 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         path = image.get('path', None)
         if path is not None:
             path = os.path.abspath(path)
-            name = self.fv.name_image_from_path(path, idx=idx)
+            name = iohelper.name_image_from_path(path, idx=idx)
         else:
             name = 'NoName'
 
@@ -687,11 +732,10 @@ class Thumbs(GingaPlugin.GlobalPlugin):
     def _regen_thumb_image(self, image, viewer):
         self.logger.debug("generating new thumbnail")
 
-        if not isinstance(image, BaseImage.BaseImage):
+        if not self.thumb_generator.viewable(image):
             # this is not a regular image type
-            image = RGBImage.RGBImage()
             tmp_path = os.path.join(icondir, 'fits.png')
-            image.load_file(tmp_path)
+            image = self.rgb_opener.load_file(tmp_path)
 
         self.thumb_generator.set_image(image)
         if viewer is not None:
@@ -735,14 +779,12 @@ class Thumbs(GingaPlugin.GlobalPlugin):
             except Exception as e:
                 self.logger.warning("Error generating thumbnail: %s" % (str(e)))
 
-        thmb_image = RGBImage.RGBImage()
-        thmb_image.set(name=info.name, placeholder=False)
-
         # Choice [C]: is there a cached thumbnail image on disk we can use?
         if (thumbpath is not None) and os.path.exists(thumbpath):
             try:
                 # try to load the thumbnail image
-                thmb_image.load_file(thumbpath)
+                thmb_image = self.rgb_opener.load_file(thumbpath)
+                thmb_image.set(name=info.name, placeholder=False)
                 thmb_image = self._regen_thumb_image(thmb_image, None)
                 thumb_extra.rgbimg = thmb_image
                 return thmb_image
@@ -752,8 +794,8 @@ class Thumbs(GingaPlugin.GlobalPlugin):
 
         # Choice [D]: load a placeholder image
         tmp_path = os.path.join(icondir, 'fits.png')
-        thmb_image.load_file(tmp_path)
-        thmb_image.set(path=None, placeholder=True)
+        thmb_image = self.rgb_opener.load_file(tmp_path)
+        thmb_image.set(name=info.name, path=None, placeholder=True)
 
         return thmb_image
 
@@ -821,6 +863,8 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         return (xt, yt, xi, yi)
 
     def get_visible_thumbs(self):
+        if not self.gui_up:
+            return []
         x1, y1, x2, y2 = self.c_view.get_datarect()
         self.logger.debug("datarect=(%f, %f, %f, %f)", x1, y1, x2, y2)
 
@@ -840,6 +884,8 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         return thumbs
 
     def add_visible_thumbs(self):
+        if not self.gui_up:
+            return
         canvas = self.c_view.get_canvas()
 
         with self.thmblock:
@@ -847,6 +893,7 @@ class Thumbs(GingaPlugin.GlobalPlugin):
             old_thumb_keys = set(self._displayed_thumb_dict.keys())
             if old_thumb_keys == thumb_keys:
                 # no need to do anything
+                self.c_view.redraw(whence=0)
                 return
 
             to_delete = old_thumb_keys - thumb_keys
@@ -897,7 +944,7 @@ class Thumbs(GingaPlugin.GlobalPlugin):
             text = self._mk_tooltip_text(metadata)
             thumb_extra.tooltip = text
 
-        canvas = self.c_view.get_canvas()
+        canvas = self.thumb_generator.get_canvas()
         fg = self.settings.get('label_font_color', 'black')
         fontsize = self.settings.get('label_font_size', 10)
 
@@ -982,6 +1029,8 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         Clears the thumbnail display widget of all thumbnails, but does
         not remove them from the thumb_dict or thumb_list.
         """
+        if not self.gui_up:
+            return
         canvas = self.c_view.get_canvas()
         canvas.delete_all_objects()
         self.c_view.redraw(whence=0)
@@ -1011,7 +1060,8 @@ class Thumbs(GingaPlugin.GlobalPlugin):
         if xi is not None:
             xi += self.thumb_width * 2
             xm, ym, x_, y_ = self._calc_thumb_pos(0, 0)
-            self.c_view.set_limits([(xm, ym), (xi, yi)], coord='data')
+            if self.gui_up:
+                self.c_view.set_limits([(xm, ym), (xi, yi)], coord='data')
 
         if new_thumbkey is not None:
             self.auto_scroll(new_thumbkey)
@@ -1075,7 +1125,8 @@ class Thumbs(GingaPlugin.GlobalPlugin):
             bnch.image.image = thmb_image
             thumb_extra.rgbimg = thmb_image
 
-            self.c_view.redraw(whence=0)
+            if self.gui_up:
+                self.c_view.redraw(whence=0)
             self.logger.debug("update finished.")
 
     def __str__(self):
