@@ -4,10 +4,6 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
-from __future__ import absolute_import
-
-from ginga.util import six
-
 import PIL.Image as PILimage
 
 have_pil_imagetk = False
@@ -16,57 +12,46 @@ try:
     from PIL.ImageTk import PhotoImage
     have_pil_imagetk = True
 except ImportError:
-    if six.PY2:
-        from Tkinter import PhotoImage
-    else:
-        from tkinter import PhotoImage
+    from tkinter import PhotoImage
 
-from ginga import Mixins, Bindings  # noqa
+from ginga import ImageView, Mixins, Bindings  # noqa
 from ginga.canvas.mixins import DrawingMixin, CanvasMixin, CompoundMixin  # noqa
+from ginga.canvas import render
 from ginga.util.toolbox import ModeIndicator  # noqa
 
 from . import TkHelp  # noqa
 
-try:
-    # See if we have aggdraw module--best choice
-    from ginga.aggw.ImageViewAgg import ImageViewAgg as ImageView, \
-        ImageViewAggError as ImageViewError
 
-except ImportError:
-    try:
-        # No dice. How about the PIL module?
-        from ginga.pilw.ImageViewPil import ImageViewPil as ImageView, \
-            ImageViewPilError as ImageViewError
-
-    except ImportError:
-        try:
-            # No, hmm..ok, see if we have opencv module...
-            from ginga.cvw.ImageViewCv import ImageViewCv as ImageView, \
-                ImageViewCvError as ImageViewError
-
-        except ImportError:
-            # Fall back to mock--there will be no graphic overlays
-            from ginga.mockw.ImageViewMock import ImageViewMock as ImageView, \
-                ImageViewMockError as ImageViewError
-
-
-class ImageViewTkError(ImageViewError):
+class ImageViewTkError(ImageView.ImageViewError):
     pass
 
 
-class ImageViewTk(ImageView):
+class ImageViewTk(ImageView.ImageViewBase):
 
     def __init__(self, logger=None, rgbmap=None, settings=None):
-        ImageView.__init__(self, logger=logger,
-                           rgbmap=rgbmap,
-                           settings=settings)
+        ImageView.ImageViewBase.__init__(self, logger=logger,
+                                         rgbmap=rgbmap,
+                                         settings=settings)
 
         self.tkcanvas = None
         self.tkphoto = None
 
+        self.t_.set_defaults(renderer='cairo')
+
         self._defer_task = None
         self.msgtask = None
 
+        self.rgb_order = 'RGBA'
+
+        self.renderer = None
+        # Pick a renderer that can work with us
+        renderers = ['cairo', 'agg', 'pil', 'opencv']
+        preferred = self.t_['renderer']
+        if preferred in renderers:
+            renderers.remove(preferred)
+        self.possible_renderers = [preferred] + renderers
+        self.choose_best_renderer()
+
     def set_widget(self, canvas):
         """Call this method with the Tkinter canvas that will be used
         for the display.
@@ -90,6 +75,26 @@ class ImageViewTk(ImageView):
     def get_widget(self):
         return self.tkcanvas
 
+    def choose_renderer(self, name):
+        klass = render.get_render_class(name)
+        self.renderer = klass(self)
+
+        if self.tkcanvas is not None:
+            wd = self.tkcanvas.winfo_width()
+            ht = self.tkcanvas.winfo_height()
+            self.configure_window(wd, ht)
+
+    def choose_best_renderer(self):
+        for name in self.possible_renderers:
+            try:
+                self.choose_renderer(name)
+                self.logger.info("best renderer available is '{}'".format(name))
+                return
+            except Exception as e:
+                continue
+
+        raise ImageViewTkError("No valid renderers available: {}".format(str(self.possible_renderers)))
+
     def update_image(self):
         if self.tkcanvas is None:
             return
@@ -270,7 +275,7 @@ class ImageViewEvent(ImageViewTk):
         except KeyError:
             return keyname
 
-    def get_keyTable(self):
+    def get_key_table(self):
         return self._keytbl
 
     def focus_event(self, event, hasFocus):
@@ -404,7 +409,7 @@ class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
                                 settings=settings)
         Mixins.UIMixin.__init__(self)
 
-        self.ui_setActive(True)
+        self.ui_set_active(True)
 
         if bindmap is None:
             bindmap = ImageViewZoom.bindmapClass(self.logger)
