@@ -6,12 +6,9 @@
 #
 """
 There are two possible choices for a python FITS file reading package
-compatible with Ginga: astropy/pyfits and fitsio.  Both are based on
-the CFITSIO library, although it seems that astropy's version has
-changed quite a bit from the original, while fitsio is still tracking
-the current version.
+compatible with Ginga: astropy and fitsio.
 
-To force the use of one, do:
+To force the use of one programmatically, do:
 
     from ginga.util import io_fits
     io_fits.use('package')
@@ -19,15 +16,30 @@ To force the use of one, do:
 (replace 'package' with one of {'astropy', 'fitsio'}) before you load
 any images.  Otherwise Ginga will try to pick one for you.
 """
+import re
 import numpy as np
 
+from ginga.AstroImage import AstroImage, AstroHeader
+from ginga.table.AstroTable import AstroTable
+
 from ginga.misc import Bunch
 from ginga.util import iohelper
+from ginga.util.io import io_base
 
 fits_configured = False
 fitsLoaderClass = None
-have_astropy = False
-have_fitsio = False
+
+try:
+    from astropy.io import fits as pyfits
+    have_astropy = True
+except ImportError:
+    have_astropy = False
+
+try:
+    import fitsio
+    have_fitsio = True
+except ImportError as e:
+    have_fitsio = False
 
 
 class FITSError(Exception):
@@ -35,61 +47,170 @@ class FITSError(Exception):
 
 
 def use(fitspkg, raise_err=True):
-    global fits_configured, fitsLoaderClass, have_astropy, pyfits, \
-        have_fitsio, fitsio
+    global fits_configured, fitsLoaderClass
 
     if fitspkg == 'astropy':
-        try:
-            from astropy.io import fits as pyfits
-            have_astropy = True
+        if have_astropy:
             fitsLoaderClass = PyFitsFileHandler
+            fits_configured = True
             return True
 
-        except ImportError:
-            if raise_err:
-                raise
+        elif raise_err:
+            raise("Error importing 'astropy.io.fits'; "
+                  "please check installation")
 
         return False
 
     elif fitspkg == 'fitsio':
-        try:
-            import fitsio
-            have_fitsio = True
+        if have_fitsio:
             fitsLoaderClass = FitsioFileHandler
+            fits_configured = True
             return True
 
-        except ImportError as e:
-            if raise_err:
-                raise
+        elif raise_err:
+            raise("Error importing 'fitsio'; "
+                  "please check installation")
+
         return False
 
     return False
 
 
-class BaseFitsFileHandler(object):
+class BaseFitsFileHandler(io_base.BaseIOHandler):
+
+    # TODO: remove in a future version
+    @classmethod
+    def register_type(cls, name, klass):
+        raise Exception("This method has been deprecated;"
+                        "you don't need to call it anymore.")
 
     def __init__(self, logger):
-        super(BaseFitsFileHandler, self).__init__()
+        super(BaseFitsFileHandler, self).__init__(logger)
 
-        self.logger = logger
         self.fileinfo = None
         self.fits_f = None
         self.hdu_info = []
         self.hdu_db = {}
         self.extver_db = {}
-        self.factory_dict = {}
-
-    def register_type(self, name, klass):
-        self.factory_dict[name.lower()] = klass
 
     def get_factory(self):
         hdlr = self.__class__(self.logger)
-        hdlr.factory_dict.update(self.factory_dict)
         return hdlr
 
+    def __len__(self):
+        return len(self.hdu_info)
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+        return False
+
+    def load_idx(self, idx, **kwargs):
+        if len(self) == 0:
+            raise ValueError("Please call open_file() first!")
+
+        return self.get_hdu(idx, **kwargs)
+
+    def load_idx_cont(self, idx_spec, loader_cont_fn, **kwargs):
+        """
+        Parameters
+        ----------
+        idx_spec : str
+            A string in the form of a pair of brackets enclosing some
+            index expression matching HDUs in the file
+
+        loader_cont_fn : func (data_obj) -> None
+            A loader continuation function that takes a data object
+            generated from loading an HDU and does something with it
+
+        kwargs : optional keyword arguments
+            Any optional keyword arguments are passed to the code that
+            loads the data from the file
+        """
+        if len(self) == 0:
+            raise ValueError("Please call open_file() first!")
+
+        idx_lst = list(self.get_matching_indexes(idx_spec))
+        if len(idx_lst) == 0:
+            raise ValueError("Spec {} matches no data objects in file".format(idx_spec))
+
+        for idx in idx_lst:
+            try:
+                dst_obj = self.load_idx(idx, **kwargs)
+
+                loader_cont_fn(dst_obj)
+
+            except Exception as e:
+                self.logger.error("Error loading index '%s': %s" % (idx, str(e)),
+                                  exc_info=True)
+
+    def get_matching_indexes(self, idx_spec):
+        """
+        Parameters
+        ----------
+        idx_spec : str
+            A string in the form of a pair of brackets enclosing some
+            index expression matching HDUs in the file
+
+        Returns
+        -------
+        result : list
+            A list of HDU indexes that can be used to access each HDU
+            matching the pattern
+        """
+        # if index is missing, assume to open the first HDU we know how
+        # to do something with
+        if idx_spec is None or idx_spec == '':
+            idx, hdu = self.find_first_good_hdu()
+            return [idx]
+
+        if isinstance(idx_spec, int):
+            return [idx_spec]
+
+        match = re.match(r'^\[(.+)\]$', idx_spec)
+        if not match:
+            return []
+
+        idx_spec = match.group(1).strip()
+        if ',' in idx_spec:
+            name, extver = idx_spec.split(',')
+            name, extver = name.strip(), extver.strip()
+        else:
+            name, extver = idx_spec.strip(), None
+
+        name = name.upper()
+
+        if extver is None:
+            if re.match(r'^\d+$', name):
+                # index just names a single HDU by number
+                return [int(name)]
+
+            # <-- assume name is an HDU name and extver is 1
+            extver = '1'
+
+        # find all HDU's matching the name and extver
+        idx_lst = []
+        idx = 0
+        for info in self.hdu_info:
+            if name == '*' or name == info.name:
+                if extver == '*' or extver == str(info.extver):
+                    idx_lst.append(info.index)
+
+        return idx_lst
+
+    def get_directory(self):
+        return self.hdu_db
+
+    def get_info_idx(self, idx):
+        return self.hdu_db[idx]
+
 
 class PyFitsFileHandler(BaseFitsFileHandler):
 
+    name = 'astropy.fits'
+
     def __init__(self, logger):
         if not have_astropy:
             raise FITSError(
@@ -112,47 +233,58 @@ class PyFitsFileHandler(BaseFitsFileHandler):
                     continue
                 ahdr.set_card(card.key, card.value, comment=card.comment)
 
-    def load_hdu(self, hdu, dstobj=None, **kwargs):
-
+    def get_hdu_type(self, hdu):
         if isinstance(hdu, (pyfits.ImageHDU,
                             pyfits.CompImageHDU,
                             pyfits.PrimaryHDU,
                             )):
-            # <-- data is an image
-
-            if dstobj is None:
-                # get model class for this type of object
-                obj_class = self.factory_dict.get('image', None)
-                if obj_class is None:
-                    raise FITSError(
-                        "I don't know how to load objects of kind 'image'")
-
-                dstobj = obj_class(logger=self.logger)
-
-            # For now, call back into the object to load it from pyfits-style
-            # HDU in future migrate to storage-neutral format
-            dstobj.load_hdu(hdu, **kwargs)
+            return 'image'
 
         elif isinstance(hdu, (pyfits.TableHDU,
                               pyfits.BinTableHDU)):
-            # <-- data is a table
+            return 'table'
+
+        return None
+
+    def load_hdu(self, hdu, dstobj=None, **kwargs):
+
+        typ = self.get_hdu_type(hdu)
+        if typ == 'image':
 
             if dstobj is None:
-                # get model class for this type of object
-                obj_class = self.factory_dict.get('table', None)
-                if obj_class is None:
-                    raise FITSError(
-                        "I don't know how to load objects of kind 'table'")
+                dstobj = AstroImage(logger=self.logger)
+                dstobj.load_hdu(hdu, **kwargs)
 
-                dstobj = obj_class(logger=self.logger)
+            else:
+                # TODO: migrate code from AstroImage to here
+                dstobj.load_hdu(hdu, **kwargs)
 
-            # For now, call back into the object to load it from pyfits-style
-            # HDU in future migrate to storage-neutral format
+        elif typ == 'table':
+            # <-- data may be a table
+
+            if hdu.name == 'ASDF':
+                if dstobj is not None:
+                    raise ValueError("It is not supported to load ASDF HDU with dstobj != None")
+
+                self.logger.debug('Attempting to load {} extension from '
+                                  'FITS'.format(hdu.name))
+                from ginga.util.io import io_asdf
+                opener = io_asdf.ASDFFileHandler(self.logger)
+                return opener.load_asdf_hdu_in_fits(self.fits_f, hdu, **kwargs)
+
+            if dstobj is None:
+                dstobj = AstroTable(logger=self.logger)
+
+            self.logger.debug('Attempting to load table from FITS')
+
+            # TODO: migrate code from AstroTable to here
             dstobj.load_hdu(hdu, **kwargs)
 
         else:
             raise FITSError("I don't know how to read this HDU")
 
+        dstobj.io = self
+
         return dstobj
 
     def load_file(self, filespec, numhdu=None, dstobj=None, memmap=None,
@@ -216,12 +348,7 @@ class PyFitsFileHandler(BaseFitsFileHandler):
             # lookups by numerical index or (NAME, EXTVER)
             d = Bunch.Bunch(index=idx, name=name, extver=extver)
             if len(tup) > 5:
-                import astropy
-                from astropy.utils.introspection import minversion
-                if minversion(astropy, '2.0'):
-                    d.setvals(htype=tup[3], dtype=tup[6])
-                else:
-                    d.setvals(htype=tup[2], dtype=tup[5])
+                d.setvals(htype=tup[3], dtype=tup[6])
             self.hdu_info.append(d)
             # different ways of accessing this HDU:
             # by numerical index
@@ -233,79 +360,82 @@ class PyFitsFileHandler(BaseFitsFileHandler):
             idx += 1
 
         self.extver_db = extver_db
+        return self
 
     def close(self):
         self.hdu_info = None
         self.hdu_db = {}
         self.extver_db = {}
         self.info = None
+        fits_f = self.fits_f
         self.fits_f = None
+        fits_f.close()
 
-    def __len__(self):
-        return len(self.hdu_info)
+    def find_first_good_hdu(self):
+
+        found_valid_hdu = False
+        for i, d in enumerate(self.hdu_info):
+            name = d.name
+            hdu = self.fits_f[i]
+
+            # rule out HDUs we can't deal with
+            typ = self.get_hdu_type(hdu)
+            if typ not in ('image', 'table'):
+                continue
+
+            if not isinstance(hdu.data, np.ndarray):
+                # We need to open a numpy array
+                continue
+
+            if 0 in hdu.data.shape:
+                # non-pixel or zero-length data hdu?
+                continue
+
+            # Looks good, let's try it
+            found_valid_hdu = True
+            extver = d.extver
+            _numhdu = (name, extver)
+            if (len(name) == 0) or (_numhdu not in self.fits_f):
+                numhdu = i
+            else:
+                numhdu = _numhdu
+            break
+
+        if not found_valid_hdu:
+            # Load just the header
+            hdu = self.fits_f[0]
+            d = self.hdu_info[0]
+            name = d.name
+            extver = d.extver
+            _numhdu = (name, extver)
+            if (len(name) == 0) or (_numhdu not in self.fits_f):
+                numhdu = 0
+            else:
+                numhdu = _numhdu
+
+        return (numhdu, hdu)
 
     def get_hdu(self, numhdu, dstobj=None, **kwargs):
 
         if numhdu is None:
-            found_valid_hdu = False
-            for i, d in enumerate(self.hdu_info):
-                name = d.name
-                hdu = self.fits_f[i]
+            numhdu, hdu = self.find_first_good_hdu()
 
-                # rule out HDUs we can't deal with
-                if not isinstance(hdu, (pyfits.ImageHDU,
-                                        pyfits.CompImageHDU,
-                                        pyfits.PrimaryHDU,
-                                        pyfits.TableHDU,
-                                        pyfits.BinTableHDU,
-                                        )):
-                    continue
+        elif numhdu in self.hdu_db:
+            d = self.hdu_db[numhdu]
+            hdu = self.fits_f[d.index]
+            # normalize the index
+            numhdu = (d.name, d.extver)
 
-                if not isinstance(hdu.data, np.ndarray):
-                    # We need to open a numpy array
-                    continue
-
-                if 0 in hdu.data.shape:
-                    # non-pixel or zero-length data hdu?
-                    continue
-
-                # Looks good, let's try it
-                found_valid_hdu = True
-                extver = d.extver
-                _numhdu = (name, extver)
-                if (len(name) == 0) or (_numhdu not in self.fits_f):
-                    numhdu = i
-                else:
-                    numhdu = _numhdu
-                break
-
-            if not found_valid_hdu:
-                # Load just the header
-                hdu = self.fits_f[0]
-                d = self.hdu_info[0]
-                name = d.name
-                extver = d.extver
-                _numhdu = (name, extver)
-                if (len(name) == 0) or (_numhdu not in self.fits_f):
-                    numhdu = 0
-                else:
-                    numhdu = _numhdu
-
-        elif isinstance(numhdu, (int, str)):
+        else:
             hdu = self.fits_f[numhdu]
+            # normalize the hdu index, if possible
             name = hdu.name
             extver = hdu.ver
             _numhdu = (name, extver)
-            if (len(name) > 0) and (_numhdu in self.fits_f):
+            if ((len(name) > 0) and (_numhdu in self.fits_f) and
+                hdu is self.fits_f[_numhdu]):
                 numhdu = _numhdu
 
-        self.logger.debug("HDU index looks like: %s" % str(numhdu))
-        if numhdu not in self.fits_f:
-            info = self.hdu_db[numhdu]
-            hdu = self.fits_f[info.index]
-        else:
-            hdu = self.fits_f[numhdu]
-
         dstobj = self.load_hdu(hdu, dstobj=dstobj, fobj=self.fits_f,
                                **kwargs)
 
@@ -345,6 +475,8 @@ class PyFitsFileHandler(BaseFitsFileHandler):
 
 class FitsioFileHandler(BaseFitsFileHandler):
 
+    name = 'fitsio'
+
     def __init__(self, logger):
         if not have_fitsio:
             raise FITSError(
@@ -356,6 +488,8 @@ class FitsioFileHandler(BaseFitsFileHandler):
                          fitsio.BINARY_TBL: 'BinaryTBL',
                          fitsio.ASCII_TBL: 'AsciiTBL',
                          }
+        self.hdu_info = []
+        self.hdu_db = {}
 
     def fromHDU(self, hdu, ahdr):
         header = hdu.read_header()
@@ -364,37 +498,43 @@ class FitsioFileHandler(BaseFitsFileHandler):
                 continue
             ahdr.set_card(d['name'], d['value'], comment=d.get('comment', ''))
 
-    def load_hdu(self, hdu, dstobj=None, **kwargs):
-        from ginga import AstroImage  # Put here to avoid circular import
-
+    def get_hdu_type(self, hdu):
         hduinfo = hdu.get_info()
         hdutype = hduinfo.get('hdutype', None)
-
-        ahdr = AstroImage.AstroHeader()
-        self.fromHDU(hdu, ahdr)
-
-        metadata = dict(header=ahdr)
-        data = hdu.read()
-
         if hdutype == fitsio.IMAGE_HDU:
-            # <-- data is an image
-
-            if dstobj is None:
-                # get model class for this type of object
-                obj_class = self.factory_dict.get('image', None)
-                if obj_class is None:
-                    raise FITSError(
-                        "I don't know how to load objects of kind 'image'")
-
-                dstobj = obj_class(logger=self.logger)
-
-            dstobj.load_data(data, metadata=metadata)
+            return 'image'
 
         elif hdutype in (fitsio.ASCII_TBL, fitsio.BINARY_TBL):
+            return 'table'
+
+        return None
+
+    def load_hdu(self, hdu, dstobj=None, **kwargs):
+        typ = self.get_hdu_type(hdu)
+
+        if typ == 'image':
+            # <-- data is an image
+            ahdr = AstroHeader()
+            self.fromHDU(hdu, ahdr)
+
+            metadata = dict(header=ahdr)
+
+            data = hdu.read()
+
+            if dstobj is None:
+                dstobj = AstroImage(logger=self.logger)
+                dstobj.load_data(data, metadata=metadata)
+
+            else:
+                dstobj.load_data(data, metadata=metadata)
+
+        elif typ == 'table':
             # <-- data is a table
             raise FITSError(
                 "FITS tables are not yet readable using ginga/fitsio")
 
+        dstobj.io = self
+
         return dstobj
 
     def load_file(self, filespec, numhdu=None, dstobj=None, memmap=None,
@@ -432,7 +572,10 @@ class FitsioFileHandler(BaseFitsFileHandler):
             hdu = fits_f[idx]
             hduinfo = hdu.get_info()
             name = hduinfo['extname']
-            extver = hduinfo['extver']
+            # figure out the EXTVER for this HDU
+            #extver = hduinfo['extver']
+            extver = extver_db.setdefault(name, 0)
+            extver += 1
             extver_db[name] = extver
 
             # prepare a record of pertinent info about the HDU for
@@ -450,71 +593,82 @@ class FitsioFileHandler(BaseFitsFileHandler):
             if len(name) > 0 and extver >= 0 and key not in self.hdu_db:
                 self.hdu_db[key] = d
 
+        return self
+
     def close(self):
-        self.hdu_info = None
+        self.hdu_info = []
         self.hdu_db = {}
         self.extver_db = {}
         self.info = None
         self.fits_f = None
 
-    def __len__(self):
-        return len(self.hdu_info)
+    def find_first_good_hdu(self):
+
+        found_valid_hdu = False
+        for i, d in enumerate(self.hdu_info):
+            name = d.name
+            extver = d.extver
+            hdu = self.fits_f[i]
+            hduinfo = hdu.get_info()
+
+            if not ('ndims' in hduinfo) or (hduinfo['ndims'] == 0):
+                # compressed FITS file or non-pixel data hdu?
+                continue
+
+            if not hasattr(hdu, 'read'):
+                continue
+            data = hdu.read()
+
+            if not isinstance(data, np.ndarray):
+                # We need to open a numpy array
+                continue
+
+            if 0 in data.shape:
+                # non-pixel or zero-length data hdu?
+                continue
+
+            # Looks good, let's try it
+            found_valid_hdu = True
+            if len(name) == 0:
+                numhdu = i
+            else:
+                numhdu = (name, extver)
+            break
+
+        if not found_valid_hdu:
+            # Just load the header
+            hdu = self.fits_f[0]
+            d = self.hdu_info[0]
+            name = d.name
+            extver = d.extver
+            if len(name) == 0:
+                numhdu = 0
+            else:
+                numhdu = (name, extver)
+
+        return numhdu, hdu
 
     def get_hdu(self, numhdu, dstobj=None, **kwargs):
 
         if numhdu is None:
-            found_valid_hdu = False
-            for i, d in enumerate(self.hdu_info):
-                name = d.name
-                extver = d.extver
-                hdu = self.fits_f[i]
-                hduinfo = hdu.get_info()
+            numhdu, hdu = self.find_first_good_hdu()
 
-                if not ('ndims' in hduinfo) or (hduinfo['ndims'] == 0):
-                    # compressed FITS file or non-pixel data hdu?
-                    continue
+        elif numhdu in self.hdu_db:
+            d = self.hdu_db[numhdu]
+            hdu = self.fits_f[d.index]
+            # normalize the index
+            numhdu = (d.name, d.extver)
 
-                if not hasattr(hdu, 'read'):
-                    continue
-                data = hdu.read()
-
-                if not isinstance(data, np.ndarray):
-                    # We need to open a numpy array
-                    continue
-
-                if 0 in data.shape:
-                    # non-pixel or zero-length data hdu?
-                    continue
-
-                # Looks good, let's try it
-                found_valid_hdu = True
-                if len(name) == 0:
-                    numhdu = i
-                else:
-                    numhdu = (name, extver)
-                break
-
-            if not found_valid_hdu:
-                # Just load the header
-                hdu = self.fits_f[0]
-                d = self.hdu_info[0]
-                name = d.name
-                extver = d.extver
-                if len(name) == 0:
-                    numhdu = 0
-                else:
-                    numhdu = (name, extver)
-
-        elif isinstance(numhdu, (int, str)):
+        else:
             hdu = self.fits_f[numhdu]
+            # normalize the hdu index, if possible
             hduinfo = hdu.get_info()
             name = hduinfo['extname']
             extver = hduinfo['extver']
-            if len(name) > 0:
-                numhdu = (name, extver)
-
-        self.logger.debug("HDU index looks like: %s" % str(numhdu))
-        hdu = self.fits_f[numhdu]
+            _numhdu = (name, extver)
+            if ((len(name) > 0) and (_numhdu in self.fits_f) and
+                hdu is self.fits_f[_numhdu]):
+                numhdu = _numhdu
 
         dstobj = self.load_hdu(hdu, dstobj=dstobj, fobj=self.fits_f,
                                **kwargs)
@@ -556,17 +710,31 @@ class FitsioFileHandler(BaseFitsFileHandler):
 
 
 if not fits_configured:
-    # default
-    fitsLoaderClass = PyFitsFileHandler
+    if have_astropy:
+        # default
+        fitsLoaderClass = PyFitsFileHandler
 
-    # try to use them in this order
-    # astropy is faster
-    for name in ('astropy', 'fitsio'):
-        if use(name, raise_err=False):
-            break
+    elif have_fitsio:
+        fitsLoaderClass = FitsioFileHandler
 
 
 def get_fitsloader(kind=None, logger=None):
+    if kind is not None:
+        if kind == 'fitsio':
+            return FitsioFileHandler(logger)
+        else:
+            return PyFitsFileHandler(logger)
+
     return fitsLoaderClass(logger)
 
+
+def load_file(filepath, idx=None, logger=None, **kwargs):
+    """
+    Load an object from a FITS file.
+
+    """
+    opener = get_fitsloader(logger=logger)
+    return opener.load_file(filepath, numhdu=idx, **kwargs)
+
+
 # END
